using System;
using System.Configuration;
using System.Data;
using System.Text;
using System.Xml;
using EnterpriseLibrary.Extensions.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Storage;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Threading;

namespace EnterpriseLibrary.Extensions
{
    public class DatabaseStorageProvider : StorageProvider, IStorageProviderWriter
    {
        private RuntimeConfigurationView runtimeConfigurationView;

        public DatabaseStorageProvider() : base()
        {
        }

        public override IConfigurationChangeWatcher CreateConfigurationChangeWatcher()
        {
            DatabaseStorageProviderData databaseStorageProviderData = GetDatabaseStorageProviderData();
            return new ConfigurationChangeDatabaseWatcher(databaseStorageProviderData.DatabaseInstance, databaseStorageProviderData.GetStoredProcedure, databaseStorageProviderData.SetStoredProcedure, CurrentSectionName, runtimeConfigurationView) ;
        }

        public override void Initialize(ConfigurationView configurationView)
        {
            ArgumentValidation.CheckForNullReference(configurationView, "configurationView");

            GetStorageCursor(configurationView);
        }

        public override object Read()
        {
        	object data = LoadData();
			if (data is XmlDocument)
			{
				XmlDocument xmlApplicationDocument = data as XmlDocument;
				if (xmlApplicationDocument == null) return null;
				XmlNode sectionNode = xmlApplicationDocument.SelectSingleNode(CurrentSectionName);
				if (sectionNode == null)
				{
					throw new ConfigurationException(SR.ExceptionXmlStorageSectionNotFoundError(CurrentSectionName, GetStoredProcedure));
				}
				sectionNode = sectionNode.CloneNode(true);
				// since we wrapped the section name here in the storage provider
				return sectionNode.FirstChild;
			}
        	else
				return data;
        }

        public void Write(object value)
        {
            ArgumentValidation.CheckForNullReference(value, "value");
			if (value is XmlNode)
			{
				XmlNode valueNode = (XmlNode)value;
				XmlDocument xmlDoc = new XmlDocument();
				CreateXmlDeclaration(xmlDoc);
				XmlNode sectionNode = xmlDoc.CreateElement(CurrentSectionName);
				xmlDoc.AppendChild(sectionNode);
				CloneNode(valueNode, sectionNode);
				SaveData(xmlDoc);
			}
			else if (value is byte[])
			{
				SaveData((byte[])value);
			}
        }

        private object LoadData()
        {
            string data;
        	string type;
			
			Database db;
			try
			{
				db = DatabaseFactory.CreateDatabase(DatabaseInstance);
			}
			// when we use the Config Console to modify this information we 
			// have to dynamically build the database factory on which we depend
			catch (ConfigurationException configEx)		
			{
				System.Diagnostics.Debug.WriteLine(configEx.Message);
				ConfigurationContext ctx = runtimeConfigurationView.ConfigurationContext;
				try
				{
					DatabaseProviderFactory factory = new DatabaseProviderFactory(ctx);
					db = factory.CreateDatabase(DatabaseInstance);
				}
				catch (Exception e)
				{
					throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName), e);
				}
			}
			catch (Exception e)
			{
				throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName), e);
			}

			lock(this)
			{
				try
				{
					IDataReader dataReader = db.ExecuteReader(GetStoredProcedure, CurrentSectionName);
					if( !dataReader.Read() )
						return null;
					else
					{
						type = dataReader.IsDBNull(0) ? null : dataReader.GetString(0);
						data = dataReader.IsDBNull(1) ? null : dataReader.GetString(1);
					}
					dataReader.Close();
				}
				catch (Exception e)
				{
					throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName), e);
				}
			} //end lock

            if (data == null || data.Trim().Equals(String.Empty))
            {
                throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName));
            }

        	if (type.Equals("string"))
				return DeserializeDocumentData(data);
        	else
        		return Convert.FromBase64String(data);
        }

		
    	private XmlDocument DeserializeDocumentData(string xmlData)
        {
            XmlDocument xmlDoc = null;
            try
            {
                using (ConfigurationProtector protector = runtimeConfigurationView.GetConfigurationProtector(CurrentSectionName))
                {
                    if (protector.Encrypted)
                    {
                        byte[] decryptedBytes = null;
                        byte[] encryptedBytes = null;
                        try
                        {
                            encryptedBytes = Convert.FromBase64String(xmlData);
                            decryptedBytes = protector.Decrypt(encryptedBytes);
                            xmlData = Encoding.UTF8.GetString(decryptedBytes);

                        }
                        finally
                        {
                            if (decryptedBytes != null)
                            {
                                Array.Clear(decryptedBytes, 0, decryptedBytes.Length);
                            }
                            if (encryptedBytes != null)
                            {
                                Array.Clear(encryptedBytes, 0, encryptedBytes.Length);
                            }
                        }
                    }

                    if (xmlData.Length > 0)
                    {
                        xmlDoc = new XmlDocument();
                        xmlDoc.LoadXml(xmlData);
                    }
                }
            }
            catch (XmlException e)
            {
                throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName), e);
            }

            return xmlDoc;
        }

        private string GetStoredProcedure
        {
            get
            {
				DatabaseStorageProviderData dbStorageProviderData = GetDatabaseStorageProviderData();
				return dbStorageProviderData.GetStoredProcedure;
            }
        }

        private string DatabaseInstance
        {
            get
            {
                DatabaseStorageProviderData dbStorageProviderData = GetDatabaseStorageProviderData();
                return dbStorageProviderData.DatabaseInstance;
            }
        }

        private string SetStoredProcedure
        {
            get
            {
				DatabaseStorageProviderData dbStorageProviderData = GetDatabaseStorageProviderData();
				return dbStorageProviderData.SetStoredProcedure;
            }
        }

        private DatabaseStorageProviderData GetDatabaseStorageProviderData()
        {
            StorageProviderData storageProviderData = runtimeConfigurationView.GetStorageProviderData(CurrentSectionName);
            ArgumentValidation.CheckExpectedType(storageProviderData, typeof(DatabaseStorageProviderData));
            DatabaseStorageProviderData dbStorageProviderData = (DatabaseStorageProviderData)storageProviderData;
            return dbStorageProviderData;
        }

        private void GetStorageCursor(ConfigurationView configurationView)
        {
            ArgumentValidation.CheckExpectedType(configurationView, typeof(RuntimeConfigurationView));

            runtimeConfigurationView = (RuntimeConfigurationView)configurationView;
        }

        private static void CreateXmlDeclaration(XmlDocument xmlDoc)
        {
            if (!(xmlDoc.FirstChild is XmlDeclaration))
            {
                XmlDeclaration declaration = xmlDoc.CreateXmlDeclaration("1.0", "utf-8", null);
                if (xmlDoc.HasChildNodes)
                {
                    xmlDoc.InsertBefore(declaration, xmlDoc.ChildNodes[0]);
                }
                else
                {
                    xmlDoc.AppendChild(declaration);
                }

            }
        }

        private static void CloneNode(XmlNode value, XmlNode sectionNode)
        {
            XmlNode cloneNode = sectionNode.OwnerDocument.ImportNode(value, true);
            sectionNode.AppendChild(cloneNode);
        }

    	private void SaveData(byte[] paramValue)
    	{
			Database db;
			try
			{
				db = DatabaseFactory.CreateDatabase(DatabaseInstance);
			}
				// when we use the Config Console to modify this information we have to dynamically build the database factory on which we depend
			catch (ConfigurationException configEx)		
			{
				System.Diagnostics.Debug.WriteLine(configEx.Message);
				ConfigurationContext ctx = runtimeConfigurationView.ConfigurationContext;
				try
				{
					DatabaseProviderFactory factory = new DatabaseProviderFactory(ctx);
					db = factory.CreateDatabase(DatabaseInstance);
				}
				catch (Exception e)
				{
					throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName), e);
				}
			}
			catch( Exception e )
			{
				throw new ConfigurationException(SR.ExceptionConfigurationDbCantSet(CurrentSectionName),e);
			}
					
			lock(this)
			{
				try
				{
					DBCommandWrapper cmd = db.GetStoredProcCommandWrapper(this.SetStoredProcedure);
					cmd.AddInParameter("section_name", DbType.String, CurrentSectionName);
					cmd.AddInParameter("section_type", DbType.String, "byte[]");
					cmd.AddInParameter("section_value", DbType.String, Convert.ToBase64String(paramValue));
					db.ExecuteNonQuery(cmd);
				}
				catch( Exception e )
				{
					throw new ConfigurationException(SR.ExceptionConfigurationDbCantSet(CurrentSectionName),e);
				}
			} //end lock
    	}
    	
        private void SaveData(XmlDocument xmlDoc)
        {
            using (ConfigurationProtector protector = this.runtimeConfigurationView.GetConfigurationProtector(CurrentSectionName))
            {
                string paramValue = xmlDoc.OuterXml;

                if (protector.Encrypted)
                {
                    byte[] decryptedBytes = null;
                    byte[] encryptedBytes = null;
                    try
                    {
                        decryptedBytes = Encoding.UTF8.GetBytes(paramValue);
                        encryptedBytes = protector.Encrypt(decryptedBytes);
                        paramValue = Convert.ToBase64String(encryptedBytes);
                    }
                    finally
                    {
                        if (decryptedBytes != null)
                        {
                            Array.Clear(decryptedBytes, 0, decryptedBytes.Length);
                        }
                        if (encryptedBytes != null)
                        {
                            Array.Clear(encryptedBytes, 0, encryptedBytes.Length);
                        }
                    }
                }

				Database db;
				try
				{
					db = DatabaseFactory.CreateDatabase(DatabaseInstance);
				}
					// when we use the Config Console to modify this information we have to dynamically build the database factory on which we depend
				catch (ConfigurationException configEx)		
				{
					System.Diagnostics.Debug.WriteLine(configEx.Message);
					ConfigurationContext ctx = runtimeConfigurationView.ConfigurationContext;
					try
					{
						DatabaseProviderFactory factory = new DatabaseProviderFactory(ctx);
						db = factory.CreateDatabase(DatabaseInstance);
					}
					catch (Exception e)
					{
						throw new ConfigurationException(SR.ExceptionConfigurationDbInvalidSection(CurrentSectionName), e);
					}
				}
				catch( Exception e )
				{
					throw new ConfigurationException(SR.ExceptionConfigurationDbCantSet(CurrentSectionName),e);
				}
					
				lock(this)
				{
					try
					{
						db.ExecuteNonQuery(SetStoredProcedure,CurrentSectionName,"string",paramValue);
					}
					catch( Exception e )
					{
						throw new ConfigurationException(SR.ExceptionConfigurationDbCantSet(CurrentSectionName),e);
					}
				} //end lock
			}
        }
    }
}