using System;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security.Permissions;
using System.Data;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;

namespace EnterpriseLibrary.Extensions.Expirations
{
	/// <summary>
	/// Summary description for DatabaseDependency.
	/// </summary>
	[Serializable]
	[ComVisible(false)]
	public class DatabaseDependency: ICacheItemExpiration, ISerializable
	{
		private string dependencyDatabaseInstance;
		private string dependencyTableName;
		private DateTime lastModifiedTime;

		public DatabaseDependency(string strDatabaseInstance, string strTableName)
		{
			if (Object.Equals(strTableName, null))
			{
				throw new ArgumentNullException("strTableName", SR.ExceptionNullTableName);
			}
			if (strTableName.Length == 0)
			{
				throw new ArgumentOutOfRangeException("strTableName", SR.ExceptionEmptyTableName);
			}

			if ((Object.Equals(strDatabaseInstance, null)) | strDatabaseInstance.Trim().Equals(String.Empty))
			{
				dependencyDatabaseInstance = String.Empty;
			}


			dependencyDatabaseInstance = strDatabaseInstance;
			dependencyTableName = strTableName;
			lastModifiedTime = DateTime.MinValue;
		}

		/// <summary>
		///	This method performs the deserialization of members of the current class.
		/// </summary>
		/// <param name="info">
		///	A SerializationInfo object which is deserialized by the formatter and then passed to current constructor
		/// </param>
		/// <param name="context">
		///	A StreamingContext that describes the source of the serialized stream from where the Serialization object 
		///	is retrieved
		/// </param>
		[SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
		protected DatabaseDependency(SerializationInfo info, StreamingContext context)
		{
			// Getting the value of the name of the databaseinstance, the tablename, and the last accessed time
			this.dependencyDatabaseInstance = info.GetString("databaseInstance");
			this.dependencyTableName = info.GetString("tableName");
			this.lastModifiedTime = Convert.ToDateTime(info.GetValue("lastModifiedTime", typeof(DateTime)), DateTimeFormatInfo.CurrentInfo);
		}

		/// <summary>
		/// Returns time watched file was last modified.
		/// </summary>
		public DateTime LastModifiedTime
		{
			get { return lastModifiedTime; }
		}

		public string DatabaseInstance
		{
			get { return this.dependencyDatabaseInstance;}
			set {this.dependencyDatabaseInstance = value;}
		}

		public string TableName
		{
			get { return this.dependencyTableName;}
			set {this.dependencyTableName = value;}
		}

		/// <summary>
		///	Specifies if the item has expired or not.
		/// </summary>
		/// <returns>Returns true if the item has expired, otherwise false.</returns>
		public bool HasExpired()
		{
			bool bRetVal = false;
			try
			{
				DateTime currentLastWriteTime = DateTime.MinValue;
				Database db = DatabaseFactory.CreateDatabase(dependencyDatabaseInstance);
				IDataReader dataReader = db.ExecuteReader("GetLastNotificationDate", dependencyTableName);
				if( dataReader.Read() )
					currentLastWriteTime = dataReader.IsDBNull( 0 ) ? DateTime.MinValue : dataReader.GetDateTime( 0 );
				dataReader.Close();
				if (lastModifiedTime.Equals(DateTime.MinValue))
				{
					lastModifiedTime = currentLastWriteTime;
				}

				if (lastModifiedTime.Equals(currentLastWriteTime) == false)
				{
					lastModifiedTime = currentLastWriteTime;
					bRetVal = true;
				}
			}
			catch (Exception e)
			{
				throw new ApplicationException(String.Format("{0}: {1}", SR.ExceptionInvalidDatabaseNotificationInfo(dependencyTableName),e.Message), e);
			}
			return bRetVal;

		}

		/// <summary>
		///	Notifies that the item was recently used.
		/// </summary>
		public void Notify()
		{
		}

		/// <summary>
		/// Not used
		/// </summary>
		/// <param name="owningCacheItem">Not used</param>
		public void Initialize(CacheItem owningCacheItem)
		{
		}

		/// <summary>
		///	This method performs the serialization of members of the current class.
		/// </summary>
		/// <param name="info">
		///	A SerializationInfo object which is deserialized by the formatter and then passed to current constructor
		/// </param>
		/// <param name="context">
		///	A StreamingContext that describes the source of the serialized stream from where the Serialization object 
		///	is retrieved
		/// </param>
		[SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
		public virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("databaseInstance", this.dependencyDatabaseInstance);
			info.AddValue("tableName", this.dependencyTableName);
			info.AddValue("lastModifiedTime", this.lastModifiedTime);
		}
	}
}
