using System;
using System.Data;
using XmlFileProvider;

namespace Microsoft.Practices.EnterpriseLibrary.Data.Xml
{
    /// <summary>
    /// <para>Represents data to/from an XML File.  Parameters are not supported in this version.</para>
    /// </summary>   
    public class XmlFileCommandWrapper : DBCommandWrapper
    {
        private XmlFileCommand command;
        private int rowsAffected;
        private object[] parameterValues;
        private bool needsParameters = false;
        private char parameterToken;

        internal XmlFileCommandWrapper(string commandText, CommandType commandType, char parameterToken)
        {
            this.parameterToken = parameterToken;
            this.command = CreateCommand(commandText, commandType);
        }

        internal XmlFileCommandWrapper(string commandText, CommandType commandType, char parameterToken, object[] parameterValues) : this(commandText, commandType, parameterToken)
        {
            //this.command = CreateCommand(commandText, commandType);
            this.parameterValues = parameterValues;
            if (commandType == CommandType.StoredProcedure)
            {
                this.needsParameters = true;
            }
        }

        
		public override IDbCommand Command
        {
            get { return this.command; }
        }

        public override int RowsAffected
        {
            get { return this.rowsAffected; }
            set { this.rowsAffected = value; }
        }

        public override int CommandTimeout
        {
            get { return this.command.CommandTimeout; }
            set { this.command.CommandTimeout = value; }
        }

		public override void AddParameter(string name, DbType dbType, int size, ParameterDirection direction, bool nullable, byte precision, byte scale, string sourceColumn, DataRowVersion sourceVersion, object value)
        {
            this.command.Parameters.Add(CreateParameter(name, dbType, size, direction, nullable, precision, scale, sourceColumn, sourceVersion, value));
        }

        public override void AddParameter(string name, DbType dbType, ParameterDirection direction, string sourceColumn, DataRowVersion sourceVersion, object value)
        {
            XmlFileParameter param = CreateParameter(name, dbType, 0, direction, false, 0, 0, sourceColumn, sourceVersion, value);
            this.command.Parameters.Add(param);
        }

        public override void AddOutParameter(string name, DbType dbType, int size)
        {
            AddParameter(name, dbType, size, ParameterDirection.Output, true, 0, 0, String.Empty, DataRowVersion.Default, DBNull.Value);
        }

        public override void AddInParameter(string name, DbType dbType)
        {
            AddParameter(name, dbType, ParameterDirection.Input, String.Empty, DataRowVersion.Default, null);
        }

        public override void AddInParameter(string name, DbType dbType, object value)
        {
            AddParameter(name, dbType, ParameterDirection.Input, String.Empty, DataRowVersion.Default, value);
        }

        public override void AddInParameter(string name, DbType dbType, string sourceColumn, DataRowVersion sourceVersion)
        {
            AddParameter(name, dbType, 0, ParameterDirection.Input, true, 0, 0, sourceColumn, sourceVersion, null);
        }

        public override object GetParameterValue(string name)
        {
            return name; //this.command.Parameters[BuildParameterName(name)].Value;
        }

        public override void SetParameterValue(string name, object value)
        {
            //this.command.Parameters[BuildParameterName(name)].Value = (value == null) ? DBNull.Value : value;
        }

        public override void Dispose()
        {
            this.command.Dispose();
        }

        protected override void DoDiscoverParameters(char parameterToken)
        {
			//we do not have the concept of parameters for the XMLFile.  At least not yet!! :-)
        }

        protected override void DoAssignParameterValues()
        {
			//we do not have the concept of parameters for the XMLFile.  At least not yet!! :-)
		}

        protected override bool DoIsFurtherPreparationNeeded()
        {
            return this.needsParameters;
        }

        private XmlFileParameter CreateParameter(string name, DbType type, int size, ParameterDirection direction, bool nullable, byte precision, byte scale, string sourceColumn, DataRowVersion sourceVersion, object value)
        {
            XmlFileParameter param = (XmlFileParameter)this.command.CreateParameter();
            param.ParameterName = BuildParameterName(name);

            param.DbType = type;
            param.Direction = direction;
            param.SourceColumn = sourceColumn;
            param.SourceVersion = sourceVersion;
            param.Value = (value == null) ? DBNull.Value : value;

            return param;
        }

        private bool SameNumberOfParametersAndValues()
        {
            int returnParameterCount = 1;
            int numberOfParametersToStoredProcedure = this.command.Parameters.Count - returnParameterCount;
            int numberOfValuesProvidedForStoredProcedure = this.parameterValues.Length;
            return numberOfParametersToStoredProcedure == numberOfValuesProvidedForStoredProcedure;
        }

        private XmlFileCommand CreateCommand(string commandText, CommandType commandType)
        {
            XmlFileCommand newCommand = new XmlFileCommand();
            newCommand.CommandText = commandText;
            newCommand.CommandType = commandType;

            return newCommand;
        }

        private string BuildParameterName(string name)
        {
            if (name[0] != this.parameterToken)
            {
                return name.Insert(0, new string(this.parameterToken, 1));
            }
            return name;
        }
    }
}