using System;
using System.Data;
using System.Data.Common;
using XmlFileProvider;
using System.Xml;

namespace Microsoft.Practices.EnterpriseLibrary.Data.Xml
{
    /// <summary>
    /// <para>Represents an XML File.</para>
    /// </summary>
    /// <remarks> 
    /// <para>
    /// Internally uses XMLFileProvider.
    /// </para>  
    /// </remarks>
    public class XmlFileDatabase : Microsoft.Practices.EnterpriseLibrary.Data.Database
    {
        public XmlFileDatabase() : base()
        {
        }

        protected override char ParameterToken
        {
            get { return '@'; }
        }

        public override IDbConnection GetConnection()
        {
            return new XmlFileConnection(ConnectionString);
        }

        public override DBCommandWrapper GetStoredProcCommandWrapper(string storedProcedureName)
        {
            if (storedProcedureName == null)
            {
                throw new ArgumentNullException("storedProcedureName");
            }
            if (storedProcedureName.Length == 0)
            {
                throw new ArgumentException(SR.UninitializedString, "storedProcedureName");
            }
            return new XmlFileCommandWrapper(storedProcedureName, CommandType.StoredProcedure, ParameterToken);
        }

        public override DBCommandWrapper GetStoredProcCommandWrapper(string storedProcedureName, params object[] parameterValues)
        {
            if (storedProcedureName == null)
            {
                throw new ArgumentNullException("storedProcedureName");
            }
            if (storedProcedureName.Length == 0)
            {
                throw new ArgumentException(SR.UninitializedString, "storedProcedureName");
            }
            if (parameterValues == null)
            {
                throw new ArgumentNullException("parameterValues");
            }
            return new XmlFileCommandWrapper(storedProcedureName, CommandType.StoredProcedure, ParameterToken, parameterValues);
        }

        public override DBCommandWrapper GetSqlStringCommandWrapper(string query)
        {
            if (query == null)
            {
                throw new ArgumentNullException("query");
            }
            if (query.Length == 0)
            {
                throw new ArgumentException(SR.UninitializedString, "query");
            }
            return new XmlFileCommandWrapper(query, CommandType.Text, ParameterToken);
        }

        protected override DbDataAdapter GetDataAdapter(UpdateBehavior updateBehavior, IDbConnection connection)
        {
            string queryStringToBeFilledInLater = String.Empty;
			XmlFileDataAdapter adapter = new XmlFileDataAdapter(queryStringToBeFilledInLater, (XmlFileConnection)connection);

            if (updateBehavior == UpdateBehavior.Continue)
            {
                adapter.RowUpdated += new XmlFileRowUpdatedEventHandler(OnXmlFileRowUpdated);
            }
            return adapter;
        }

        public XmlReader ExecuteXmlReader(XmlFileCommandWrapper command)
        {
            IDbConnection connection = OpenConnection();
            PrepareCommand(command, connection);

            XmlFileCommand xmlFileCommand = command.Command as XmlFileCommand;
            try
            {
                return DoExecuteXmlReader(xmlFileCommand);
            }
            catch
            {
                connection.Close();
                throw;
            }
        }

        public XmlReader ExecuteXmlReader(XmlFileCommandWrapper command, IDbTransaction transaction)
        {
            PrepareCommand(command, transaction);

            XmlFileCommand xmlFileCommand = command.Command as XmlFileCommand;
            return DoExecuteXmlReader(xmlFileCommand);
        }

        private XmlReader DoExecuteXmlReader(XmlFileCommand xmlFileCommand)
        {
            try
            {
                DateTime startTime = DateTime.Now;
                XmlReader reader = xmlFileCommand.ExecuteXmlReader();
                //Instrumentation.CommandExecuted(startTime);
                return reader;
            }
            catch
            {
                //Instrumentation.CommandFailed(xmlFileCommand.CommandText, ConnectionStringNoCredentials);
                throw;
            }
        }

        private void OnXmlFileRowUpdated(object sender, XmlFileRowUpdatedEventArgs rowThatCouldNotBeWritten)
        {
			if (rowThatCouldNotBeWritten.RecordsAffected == 0)
            {
                if (rowThatCouldNotBeWritten.Errors != null)
                {
                    rowThatCouldNotBeWritten.Row.RowError = SR.ErrorMessageUpdateDataSetRowFailure;
                    rowThatCouldNotBeWritten.Status = UpdateStatus.SkipCurrentRow;
                }
            }
        }
    }
}