using System;
using System.Data;
using System.IO;
using System.Xml;
using System.Collections;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using XmlFileBackingStore.Configuration;

namespace XmlFileBackingStore
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class XmlFileBackingStore : BaseBackingStore
	{
		private string xmlFileName;
		private string partitionName;
		private IStorageEncryptionProvider encryptionProvider;
		
		
		public XmlFileBackingStore() : base()
		{
		}

		public XmlFileBackingStore(string xmlFileName, string partitionName, IStorageEncryptionProvider encryptionProvider)
		{
			this.xmlFileName = xmlFileName;
			this.partitionName = partitionName;
			this.encryptionProvider = encryptionProvider;
		}

		/// <summary>
		/// Removes the item identified by the key from the xml file
		/// </summary>
		/// <param name="storageKey">Key of CacheItem to remove.</param>
		/// <remarks>Exceptions thrown depend on the implementation of the xml file.</remarks>
		protected override void Remove(int storageKey)
		{
			DataSet dataToLoad = XmlFileDataSet;
			if (dataToLoad.Tables.Count > 0)
			{
				string strExpression = String.Format("StorageKey = {0}", storageKey.ToString());
				DataRow[] foundRows = dataToLoad.Tables[0].Select(strExpression);
				foreach(DataRow dataRow in foundRows)
					dataRow.Delete();
				dataToLoad.AcceptChanges();
				XmlFileDataSet = dataToLoad.Copy();
			}
		}

		protected override void UpdateLastAccessedTime(int storageKey, DateTime timestamp)
		{
			DataSet dataToLoad = XmlFileDataSet;
			if (dataToLoad.Tables.Count > 0)
			{
				string strExpression = String.Format("StorageKey = {0}", storageKey.ToString());
				DataRow[] foundRows = dataToLoad.Tables[0].Select(strExpression);
				foreach(DataRow dataRow in foundRows)
					dataRow["LastAccessedTime"] = timestamp;
				dataToLoad.AcceptChanges();
				XmlFileDataSet = dataToLoad.Copy();
			}
		}

		public override void Flush()
		{
			if (File.Exists(this.xmlFileName))
				File.Delete(this.xmlFileName);
		}


		protected override void RemoveOldItem(int storageKey)
		{
			Remove(storageKey);
		}

		protected override void AddNewItem(int storageKey, CacheItem newItem)
		{
			string key = newItem.Key;

			byte[] valueBytes = SerializationUtility.ToBytes(newItem.Value);
			if (this.encryptionProvider != null)
			{
				valueBytes = this.encryptionProvider.Encrypt(valueBytes);
			}

			byte[] expirationBytes = SerializationUtility.ToBytes(newItem.Expirations);
			byte[] refreshActionBytes = SerializationUtility.ToBytes(newItem.RefreshAction);
			CacheItemPriority scavengingPriority = newItem.ScavengingPriority;
			DateTime lastAccessedTime = newItem.LastAccessedTime;

			DataSet dataToLoad = XmlFileDataSet;
			if (dataToLoad.Tables.Count == 0)
				CreateCacheItemsTable(dataToLoad);
			
			DataTable cacheItemsTable = dataToLoad.Tables[0];
			
			DataRow newDataRow = cacheItemsTable.NewRow();
			newDataRow["StorageKey"] = storageKey;
			newDataRow["Key"] = key;
			newDataRow["Value"] = (valueBytes != null) ? valueBytes : Convert.DBNull;
			newDataRow["Expirations"] = (expirationBytes != null)? expirationBytes : Convert.DBNull;
			newDataRow["RefreshAction"] = (refreshActionBytes != null)? refreshActionBytes : Convert.DBNull;
			newDataRow["ScavengingPriority"] = scavengingPriority;
			newDataRow["LastAccessedTime"] = lastAccessedTime;
			
			cacheItemsTable.Rows.Add(newDataRow);
			dataToLoad.AcceptChanges();
			
			XmlFileDataSet = dataToLoad.Copy();
		}

		protected override Hashtable LoadDataFromStore()
		{			
			DataSet dataToLoad = XmlFileDataSet;

			Hashtable dataToReturn = new Hashtable();
			if (dataToLoad.Tables.Count > 0) 
			{
				foreach (DataRow row in dataToLoad.Tables[0].Rows)
				{
					CacheItem cacheItem = CreateCacheItem(row);
					dataToReturn.Add(cacheItem.Key, cacheItem);
				}
			}
			return dataToReturn;		
		}

		private DataSet XmlFileDataSet
		{
			get
			{
				if (File.Exists(this.xmlFileName))
				{
					XmlDataDocument dataDocument = new XmlDataDocument();
					DataSet dataToLoad = dataDocument.DataSet;

					CreateCacheItemsTable(dataToLoad);
					dataDocument.Load(this.xmlFileName);
					return dataToLoad;
				}
				else
					return new DataSet();
			}
			set
			{
				try
				{
					// if theres records in the dataset, save the changes
					if ((value.Tables.Count > 0) && (value.Tables[0].Rows.Count > 0))
					{
						XmlDataDocument dataDocument = new XmlDataDocument(value);
						dataDocument.Save(this.xmlFileName);
					}
					else if(File.Exists(this.xmlFileName))	//delete the file
						File.Delete(this.xmlFileName);
				}
				catch (Exception ex)
				{
					System.Diagnostics.Debug.WriteLine(ex.Message);
					throw;
				}
			}
		}
		
		private void CreateCacheItemsTable(DataSet dataToLoad)
		{
			DataTable dataTable = dataToLoad.Tables.Add("CacheItems");
			dataTable.Columns.Add("StorageKey");
			dataTable.Columns.Add("Key");
			dataTable.Columns.Add("Value",typeof(byte[]));
			dataTable.Columns.Add("Expirations",typeof(byte[]));
			dataTable.Columns.Add("RefreshAction",typeof(byte[]));
			dataTable.Columns.Add("ScavengingPriority");
			dataTable.Columns.Add("LastAccessedTime",typeof(DateTime));				
		}

		private CacheItem CreateCacheItem(DataRow dataToLoad)
		{
			string key = (string) dataToLoad["Key"];
			object value = DeserializeValue(dataToLoad);

			CacheItemPriority scavengingPriority = (CacheItemPriority)Enum.Parse(typeof(CacheItemPriority),(string)dataToLoad["ScavengingPriority"],true);
			object refreshAction = DeserializeObject(dataToLoad, "RefreshAction");
			object expirations = SerializationUtility.ToObject((byte[]) dataToLoad["Expirations"]);
			object timestamp = (DateTime) dataToLoad["LastAccessedTime"];

			CacheItem cacheItem = new CacheItem((DateTime) timestamp, key, value, scavengingPriority, (ICacheItemRefreshAction) refreshAction, (ICacheItemExpiration[]) expirations);

			return cacheItem;
		}
		
		private object DeserializeValue(DataRow dataToLoad)
		{
			object value = dataToLoad["Value"];

			if (value == DBNull.Value)
			{
				value = null;
			}
			else
			{
				byte[] valueBytes = (byte[]) value;
				if (this.encryptionProvider != null)
				{
					valueBytes = this.encryptionProvider.Decrypt(valueBytes);
				}
				value = SerializationUtility.ToObject(valueBytes);
			}
			return value;
		}

		private object DeserializeObject(DataRow data, string columnName)
		{
			object byteArrayAsObject = data[columnName];
			if (byteArrayAsObject == DBNull.Value)
			{
				return null;
			}
			else
			{
				return SerializationUtility.ToObject((byte[]) byteArrayAsObject);
			}
		}
	
		public override int Count
		{
			get
			{
				DataSet dataToLoad = XmlFileDataSet;
				if (dataToLoad.Tables.Count > 0) 
					return dataToLoad.Tables[0].Rows.Count;
				else 
					return 0;
			}
		}

		public override void Initialize(ConfigurationView configurationView)
		{
			ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
			ArgumentValidation.CheckExpectedType(configurationView, typeof (CachingConfigurationView));

			CachingConfigurationView cachingConfigurationView = (CachingConfigurationView) configurationView;
			xmlFileCacheStorageData xmlFileConfiguration = (xmlFileCacheStorageData) cachingConfigurationView.GetCacheStorageDataForCacheManager(CurrentCacheManager);
			partitionName = xmlFileConfiguration.PartitionName;
			xmlFileName = String.Format("{0}.{1}",xmlFileConfiguration.XmlFileName,xmlFileConfiguration.PartitionName);
			if (xmlFileConfiguration.StorageEncryption != null)
			{
				StorageEncryptionFactory encryptionFactory = new StorageEncryptionFactory(cachingConfigurationView.ConfigurationContext);
				encryptionProvider = encryptionFactory.CreateSymmetricProvider(CurrentCacheManager);
			}
		}
	}
}
