using System;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Text;
using System.Reflection;

using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;

namespace ExtendedExceptionHandling
{
    /// <summary>
    /// Represents an exception formatter that formats exception
    /// objects as text.
    /// </summary>	
    public class CSVExceptionFormatter : ExceptionFormatter
    {
        private TextWriter writer;
		private char cDelimiter;

        /// <summary>
        /// Initializes a new instance of the 
        /// <see cref="TextExceptionFormatter"/> using the specified
        /// <see cref="TextWriter"/> and <see cref="Exception"/>
        /// objects.
        /// </summary>
        /// <param name="writer">The stream to write formatting information to.</param>
        /// <param name="exception">The exception to format.</param>
        public CSVExceptionFormatter(TextWriter writer, Exception exception) : base(exception)
        {
            this.writer = writer;
			Delimiter = ',';
        }

        /// <summary>
        /// Gets the underlying <see cref="TextWriter"/>
        /// that the current formatter is writing to.
        /// </summary>
        public TextWriter Writer
        {
            get { return this.writer; }
        }

        /// <summary>
        /// Writes a generic description to the underlying
        /// text stream.
        /// </summary>
        protected override void WriteDescription()
        {
            // An exception of type {0} occurred and was caught.
            // -------------------------------------------------
            string line = SR.ExceptionWasCaught(base.Exception.GetType().FullName);
			WriteWithDelimter(line);
        }

        /// <summary>
        /// Controls the formatting of the specified exception and 
        /// its inner exception if there is one.
        /// </summary>
        /// <param name="e">See <see cref="ExceptionFormatter.WriteException"/></param>
        /// <param name="outerException">See <see cref="ExceptionFormatter.WriteException"/></param>
        /// <remarks>For more information, see the documentation
        /// for the base implementation - 
        /// <see cref="ExceptionFormatter.WriteException"/></remarks>
        protected override void WriteException(Exception e, Exception outerException)
        {
			this.WriteExceptionType(e.GetType());
			this.WriteMessage(e.Message);
			this.WriteSource(e.Source);
			this.WriteHelpLink(e.HelpLink);
			this.Writer.Write('\"');
			this.WriteReflectionInfo(e);
			this.Writer.Write("\",");
			this.WriteStackTrace(e.StackTrace);
			this.Writer.WriteLine();

			Exception inner = e.InnerException;

			if (inner != null)
			{
				// recursive call
				this.WriteException(inner, e);
			}

        }

        /// <summary>
        /// Writes the current date and time to the
        /// underlying text stream.
        /// </summary>
        /// <param name="utcNow">See <see cref="ExceptionFormatter.WriteDateTime"/></param>
        protected override void WriteDateTime(DateTime utcNow)
        {
            DateTime localTime = utcNow.ToLocalTime();
            string localTimeString = localTime.ToString("G", DateTimeFormatInfo.InvariantInfo);

            this.WriteWithDelimter(localTimeString);
        }

        /// <summary>
        /// Writes the value of the <see cref="Type.AssemblyQualifiedName"/>
        /// property for the specified exception type to the underlying
        /// text stream.
        /// </summary>
        /// <param name="exceptionType">See <see cref="ExceptionFormatter.WriteExceptionType"/></param>
        protected override void WriteExceptionType(Type exceptionType)
        {
            //this.Writer.WriteLine(SR.TypeString, exceptionType.AssemblyQualifiedName);
			this.WriteWithDelimter(exceptionType.AssemblyQualifiedName);
        }

        /// <summary>
        /// Writes the value of the <see cref="Exception.Message"/>
        /// property to the underyling text stream.
        /// </summary>
        /// <param name="message">See <see cref="ExceptionFormatter.WriteMessage"/></param>
        protected override void WriteMessage(string message)
        {
            //this.Writer.WriteLine(SR.Message, message);
			this.WriteWithDelimter(message);
		}

        /// <summary>
        /// Writes the value of the specified source taken
        /// from the value of the <see cref="Exception.Source"/>
        /// property.
        /// </summary>
        /// <param name="source">See <see cref="ExceptionFormatter.WriteSource"/></param>
        protected override void WriteSource(string source)
        {
            //this.Writer.WriteLine(SR.Source, source);
			this.WriteWithDelimter(source);
        }

        /// <summary>
        /// Writes the value of the specified help link taken
        /// from the value of the <see cref="Exception.HelpLink"/>
        /// property.
        /// </summary>
        /// <param name="helpLink">The exception's help link.</param>
        protected override void WriteHelpLink(string helpLink)
        {
            //this.Writer.WriteLine(SR.HelpLink, helpLink);
			this.WriteWithDelimter(helpLink);
        }

        /// <summary>
        /// Writes the name and value of the specified property
        /// to the underlying text stream.
        /// </summary>
        /// <param name="propertyInfo">See <see cref="ExceptionFormatter.WritePropertyInfo"/></param>
        /// <param name="propertyValue">See <see cref="ExceptionFormatter.WritePropertyInfo"/></param>
        protected override void WritePropertyInfo(PropertyInfo propertyInfo, object propertyValue)
        {
            this.Writer.Write(propertyInfo.Name);
            this.Writer.Write(" : ");
            this.Writer.WriteLine(propertyValue);
			//this.WriteWithDelimter(String.Format("{0} : {1}", propertyInfo.Name, propertyValue));

        }

        /// <summary>
        /// Writes the name and value of the specified
        /// field to the underlying text stream.
        /// </summary>
        /// <param name="field">See <see cref="ExceptionFormatter.WriteFieldInfo"/></param>
        /// <param name="fieldValue">See <see cref="ExceptionFormatter.WriteFieldInfo"/></param>
        protected override void WriteFieldInfo(FieldInfo field, object fieldValue)
        {
            this.Writer.Write(field.Name);
            this.Writer.Write(" : ");
            this.Writer.WriteLine(fieldValue);
//			this.WriteWithDelimter(String.Format("{0} : {1}", field.Name, fieldValue));
		}

        /// <summary>
        /// Formats the stack trace.
        /// </summary>
        /// <param name="stackTrace">See <see cref="ExceptionFormatter.WriteStackTrace"/></param>
        /// <remarks>
        /// If there is no stack trace available, an appropriate message
        /// will be displayed.
        /// </remarks>
        protected override void WriteStackTrace(string stackTrace)
        {
			StringBuilder sbStackTrace = new StringBuilder();
			sbStackTrace.Append(SR.StackTrace);
			sbStackTrace.Append(" : ");
			if (stackTrace == null || stackTrace.Length == 0)
			{
				sbStackTrace.Append(SR.StackTraceUnavailable);
			}
			else
			{
				sbStackTrace.Append(stackTrace);
			}
			this.WriteWithDelimter(sbStackTrace.ToString());

//
//			this.Writer.Write(SR.StackTrace);
//            this.Writer.Write(" : ");
//            if (stackTrace == null || stackTrace.Length == 0)
//            {
//                this.Writer.WriteLine(SR.StackTraceUnavailable);
//            }
//            else
//            {
//                this.Writer.WriteLine(stackTrace);
//                this.Writer.WriteLine();
//            }
        }

        /// <summary>
        /// Writes the additional properties to the underlying stream.
        /// </summary>
        /// <param name="additionalInfo">See <see cref="ExceptionFormatter.WriteAdditionalInfo"/></param>
        protected override void WriteAdditionalInfo(NameValueCollection additionalInfo)
        {
			StringBuilder sbAddInfo = new StringBuilder();
			foreach (string name in additionalInfo.AllKeys)
			{
				sbAddInfo.Append(name);
				sbAddInfo.Append(" : ");
				sbAddInfo.Append(additionalInfo[name]);
				sbAddInfo.Append("\n");
			}
			this.WriteWithDelimter(sbAddInfo.ToString());

//			this.Writer.Write('\"');
//			foreach (string name in additionalInfo.AllKeys)
//            {
//                this.Writer.Write(name);
//                this.Writer.Write(" : ");
//                this.Writer.Write(additionalInfo[name]);
//                this.Writer.Write("\n");
//            }
//			this.Writer.Write("\",");
		}

		protected virtual void WriteWithDelimter(string strValue)
		{
			if (strValue == null)
				this.Writer.Write(Delimiter);
			else
			{
				string strFormat = "{0}{1}";
				char[] arrCheckDelims = new char[] {',','\n','\r'};
				if (strValue.IndexOfAny(arrCheckDelims) >= 0)
					strFormat = "\"{0}\"{1}";
				this.Writer.Write(strFormat,strValue,Delimiter);
			}
		}

		public char Delimiter
		{
			get
			{
				return cDelimiter;
			}
			set
			{
				cDelimiter = value;
			}
		}

		protected virtual void WriteHeader1()
		{

			WriteWithDelimter("Description");
			WriteWithDelimter("Date/Time");
			WriteWithDelimter("Additional Info");
			Writer.WriteLine();
		}

		protected virtual void WriteHeader2()
		{
			Writer.WriteLine();
			Writer.WriteLine();
			Writer.WriteLine();
			WriteWithDelimter("Exception Type");
			WriteWithDelimter("Message");
			WriteWithDelimter("Source");
			WriteWithDelimter("Help Link");
			WriteWithDelimter("Reflection Information");
			this.Writer.WriteLine("StackTrace");
		}

		public override void Format()
		{
			this.WriteHeader1();
			this.WriteDescription();
			this.WriteDateTime(DateTime.UtcNow);
			this.WriteAdditionalInfo(this.AdditionalInfo);
			this.WriteHeader2();
			this.WriteException(Exception, null);		
		}
	}
}