using System;
using System.Security.Permissions;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;

namespace ExtendedExceptionHandling
{
	/// <summary>
	/// Summary description for ContextualWrapHandler.
	/// </summary>
	public class ContextualWrapHandler : WrapHandler
	{
		private ContextualWrapData wrapHandlerData;

		public ContextualWrapHandler() : base()
		{
		}

		/// <summary>
		/// <para>Initializes the provider with a <see cref="ExceptionHandlingConfigurationView"/>.</para>
		/// </summary>
		/// <param name="configurationView">
		/// <para>A <see cref="ExceptionHandlingConfigurationView"/> object.</para>
		/// </param>
		public override void Initialize(ConfigurationView configurationView)
		{
			ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
			ArgumentValidation.CheckExpectedType(configurationView, typeof(ExceptionHandlingConfigurationView));

			ExceptionHandlingConfigurationView exceptionHandlingConfigurationView = (ExceptionHandlingConfigurationView)configurationView;

			ExceptionHandlerData exceptionHandlerData = exceptionHandlingConfigurationView.GetExceptionHandlerData(CurrentPolicyName, CurrentExceptionTypeName, ConfigurationName);
			ArgumentValidation.CheckExpectedType(exceptionHandlerData, typeof(WrapHandlerData));

			wrapHandlerData = (ContextualWrapData)exceptionHandlerData;
		}

		/// <summary>
		/// <para>Gets the <see cref="Type"/> of exception to wrap the original exception with.</para>
		/// </summary>
		/// <value>
		/// <para>The <see cref="Type"/> of exception to wrap the original exception with.</para>
		/// </value>
		public new Type WrapExceptionType
		{
			[ReflectionPermission(SecurityAction.Demand, MemberAccess=true)]
			get { return Type.GetType(wrapHandlerData.WrapExceptionTypeName); }
		}

		/// <summary>
		/// <para>Gets the message of the wrapped exception.</para>
		/// </summary>
		/// <value>
		/// <para>The message of the wrapped exception.</para>
		/// </value>
		public string DefaultExceptionMessage
		{
			get { return wrapHandlerData.DefaultExceptionMessage; }
		}

		/// <summary>
		/// <para>Wraps the <see cref="Exception"/> with the configuration exception type.</para>
		/// </summary>
		/// <param name="exception"><para>The exception to handle.</para></param>
		/// <param name="policyName"><para>The name of the <see cref="ExceptionPolicy"/>.</para></param>
		/// <param name="handlingInstanceId">
		/// <para>The unique ID attached to the handling chain for this handling instance.</para>
		/// </param>
		/// <returns><para>Modified exception to pass to the next handler in the chain.</para></returns>
		[ReflectionPermission(SecurityAction.Demand, MemberAccess=true)]
		public override Exception HandleException(Exception exception, string policyName, Guid handlingInstanceId)
		{
			if (! (exception is WrapperException))
				return base.HandleException(exception, policyName, handlingInstanceId);
			
			return ContextualWrapException(
				exception,
				WrapExceptionType,
				DefaultExceptionMessage, 
				handlingInstanceId);
		}

		/// <summary>
		/// Wraps an exception with a new exception of a specified type.
		/// </summary>
		/// <param name="originalException">The original exception.</param>
		/// <param name="wrapExceptionType">The type of exception to wrap the original exception with.</param>
		/// <param name="wrapExceptionMessage">The message for the new exception.</param>
		/// <returns>An exception with the innerException set to the original exception.  Returns null if unable to wrap the exception.</returns>
		private Exception ContextualWrapException(Exception originalException, Type wrapExceptionType, string defaultExceptionMessage, Guid handlingInstanceId)
		{
			string strNewExceptionMessage = originalException.Message;

			if ((strNewExceptionMessage == null) || (strNewExceptionMessage.Equals(String.Empty)))
				strNewExceptionMessage = defaultExceptionMessage;
		
			if (strNewExceptionMessage == null)
				strNewExceptionMessage = String.Empty;

			strNewExceptionMessage = ExceptionUtility.FormatExceptionMessage(strNewExceptionMessage, handlingInstanceId);

			Exception innerException = (originalException.InnerException == null)? originalException : originalException.InnerException;

			try
			{
				object[] extraParameters = new object[2];
				extraParameters[0] = strNewExceptionMessage;
				extraParameters[1] = innerException;
				return (Exception)Activator.CreateInstance(wrapExceptionType, extraParameters);
			}
			catch (Exception ex)
			{
				throw new ExceptionHandlingException(SR.ExceptionUnableToWrapException(wrapExceptionType.Name), ex);
			}
		}
	}
}
