using System;
using System.Data;
using System.Drawing.Design;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml.Serialization;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Logging.Distributor;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.Design.Client.DistributionStrategies;
using DatabaseDistributionStrategy.Configuration;

namespace DatabaseDistributionStrategy.Design
{
    /// <summary>
    /// Node representing a <see cref="DatabaseDistributionStrstegy.DatabaseDistributionStrategy"/>.
    /// </summary>
	[ServiceDependency(typeof(ILinkNodeService))]
	[ServiceDependency(typeof(IXmlIncludeTypeService))]
	[ServiceDependency(typeof(INodeCreationService))]
	public class DatabaseDistributionStrategyNode : DistributionStrategyNode
    {
		private InstanceNode instanceNode;
		private ConfigurationNodeChangedEventHandler onInstanceNodeRemoved;
		private ConfigurationNodeChangedEventHandler onInstanceNodeRenamed;
		private DatabaseDistributionStrategyData databaseDistributionStrategyData;
		private const string defaultStoredProc = "WriteLogEntry";

        /// <summary>
        /// Creates node with initial configuration data.
        /// </summary>
        public DatabaseDistributionStrategyNode() : this(new DatabaseDistributionStrategyData(SR.DatabaseDistributionStrategy,defaultStoredProc))
        {
        }

        /// <summary>
        /// Creates node with specified configuration data.
        /// </summary>
        /// <param name="databaseDistributionStrategyData">Configuration data.</param>
        public DatabaseDistributionStrategyNode(DatabaseDistributionStrategyData databaseDistributionStrategyData) : base(databaseDistributionStrategyData)
        {
			this.onInstanceNodeRemoved += new ConfigurationNodeChangedEventHandler(OnInstanceNodeRemoved);
			this.onInstanceNodeRenamed += new ConfigurationNodeChangedEventHandler(OnInstanceNodeRenamed);
			this.databaseDistributionStrategyData = databaseDistributionStrategyData;
        }

        /// <summary>
        /// Stored Proc for Writing Log Entry.
        /// </summary>
        [Required]
        [SRDescription(SR.Keys.StoredProcDescription)]
        [SRCategory(SR.Keys.CategoryGeneral)]
        public string StoredProcedure
        {
            get { return databaseDistributionStrategyData.StoredProcedure; }
            set { databaseDistributionStrategyData.StoredProcedure = value; }
        }

		[Editor(typeof(ReferenceEditor), typeof(UITypeEditor))]
		[ReferenceType(typeof(InstanceNode))]    
		[SRCategory(SR.Keys.CategoryGeneral)]
		public InstanceNode DatabaseInstance
		{
			get { return instanceNode; }
			set
			{
				ILinkNodeService service = GetService(typeof(ILinkNodeService)) as ILinkNodeService;
				instanceNode = service.CreateReference(instanceNode, value, onInstanceNodeRemoved, onInstanceNodeRenamed) as InstanceNode;
				databaseDistributionStrategyData.DatabaseInstance = string.Empty;
				if (instanceNode != null)
				{
					databaseDistributionStrategyData.DatabaseInstance = instanceNode.Name;	
				}
			}
		}

		/// <summary>
        /// The configured name.
        /// </summary>
        [ReadOnly(true)]
        public override string Name
        {
            get { return base.Name; }
            set { base.Name = value; }
        }

        /// <summary>
        /// Read only.  Returns the type name of a <see cref="DatabaseDistributionStrategy.DatabaseDistributionStrategy"/>.
        /// </summary>
        [Browsable(false)]
        public override string TypeName
        {
            get { return typeof(DatabaseDistributionStrategy).AssemblyQualifiedName; }
        }

        /// <summary>
        /// Returns the <c>TypeName</c> for visibility in the UI.
        /// </summary>
        [ReadOnly(true)]
        [SRDescription(SR.Keys.DistributionStrategyTypeNameDescription)]
        [SRCategory(SR.Keys.CategoryGeneral)]
        public string Type
        {
            get { return TypeName; }
        }

		/// <summary>
		/// See <see cref="ConfigurationNode.ResolveNodeReferences"/>.
		/// </summary>
		public override void ResolveNodeReferences()
		{
			base.ResolveNodeReferences();
			InstanceCollectionNode instanceCollectionNode = Hierarchy.FindNodeByType(typeof(InstanceCollectionNode)) as InstanceCollectionNode;
			DatabaseInstance = Hierarchy.FindNodeByName(instanceCollectionNode, databaseDistributionStrategyData.DatabaseInstance) as InstanceNode;
		}

		private void OnInstanceNodeRemoved(object sender, ConfigurationNodeChangedEventArgs e)
		{
			this.instanceNode = null;
		}

		private void OnInstanceNodeRenamed(object sender, ConfigurationNodeChangedEventArgs e)
		{
			databaseDistributionStrategyData.DatabaseInstance = e.Node.Name;
		}
    }
}