
using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Soap;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Distributor;
using Microsoft.Practices.EnterpriseLibrary.Logging.Instrumentation;
using DatabaseDistributionStrategy.Configuration;

namespace DatabaseDistributionStrategy
{
    /// <summary>
    /// DatabaseDistributionStrategy will distribute log messages asynchronously, meaning
    /// log messages will be sent to the database instead of log sinks.  The DatabaseDistributorService 
    /// will pick the messages from the database and send them to corresponding log sinks.
    /// </summary>
    public class DatabaseDistributionStrategy : ConfigurationProvider, ILogDistributionStrategy
    {
        private DatabaseDistributionStrategyData databaseDistributionStrategyData;
		private RuntimeConfigurationView configView;

        public DatabaseDistributionStrategy()
        {
        }

        /// <summary>
        /// Creates a new instance of <see cref="DatabaseDistributionStrategy"></see>
        /// </summary>
        /// <param name="databaseDistributionStrategyData">Configuration data for <see cref="DatabaseDistributionStrategy"></see></param>
        public DatabaseDistributionStrategy(DatabaseDistributionStrategyData databaseDistributionStrategyData)
        {
            ArgumentValidation.CheckForNullReference(databaseDistributionStrategyData, "databaseDistributionStrategyData");

            this.databaseDistributionStrategyData = databaseDistributionStrategyData;
        }

        /// <summary>
        /// <para>
        /// Initializes the provider with its name and any attributes associated with the provider. 
        /// </para>
        /// </summary>   
        /// <param name="configurationView">Dynamic view into the configuration information needed by the <see cref="DatabaseDistributionStrategy"></see></param>     
        public override void Initialize(ConfigurationView configurationView)
        {
            ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
            ArgumentValidation.CheckExpectedType(configurationView, typeof (LoggingConfigurationView));
			configView = new RuntimeConfigurationView(configurationView.ConfigurationContext);
            LoggingConfigurationView loggingConfigurationView = (LoggingConfigurationView) configurationView;

            DistributionStrategyData distributionStrategyData = loggingConfigurationView.GetDistributionStrategyData(ConfigurationName);
            ArgumentValidation.CheckExpectedType(distributionStrategyData, typeof (DatabaseDistributionStrategyData));

            databaseDistributionStrategyData = (DatabaseDistributionStrategyData) distributionStrategyData;
        }

        /// <summary>
        /// Send message to database for asynchronous distribution.
        /// </summary>
        /// <param name="entry">Message to log.</param>
        public void SendLog(LogEntry entry)
        {
			SoapFormatter formatter = new SoapFormatter();
            string serializedEntry = String.Empty;
            using (MemoryStream ms = new MemoryStream())
            {
                formatter.Serialize(ms, entry);
                ms.Position = 0;
                byte[] buffer = new byte[ms.Length];
                ms.Read(buffer, 0, (int) ms.Length);
                serializedEntry = Encoding.UTF8.GetString(buffer);
            }

            ExecuteStoredProc(serializedEntry);

            LoggingLogDistributedEvent.Fire(serializedEntry);
        }

        private void ExecuteStoredProc(string serializedEntry)
        {
			DatabaseProviderFactory factory = new DatabaseProviderFactory(configView.ConfigurationContext);
			Database db = (databaseDistributionStrategyData.DatabaseInstance.Equals(String.Empty))? factory.CreateDefaultDatabase()
								: factory.CreateDatabase(databaseDistributionStrategyData.DatabaseInstance);
			db.ExecuteNonQuery(databaseDistributionStrategyData.StoredProcedure,serializedEntry);
        }
    }
}
