using System;
using System.Configuration;
using System.Diagnostics;
using System.Windows.Forms;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using DatabaseDistributor.Configuration;

namespace DatabaseDistributor.Design
{
	/// <summary>
	/// Summary description for DatabaseDistributorConfigurationManager.
	/// </summary>
	public class DatabaseDistributorConfigurationManager : IConfigurationDesignManager
	{
		public DatabaseDistributorConfigurationManager()
		{
		}

		public void Register(IServiceProvider serviceProvider)
		{
			CreateCommands(serviceProvider);
			RegisterNodeMaps(serviceProvider);
		}

		private static void CreateCommands(IServiceProvider provider)
		{
			// Create the new menu item for adding the Database Distributor to an application.
			ConfigurationMenuItem item = new ConfigurationMenuItem(SR.DatabaseDistributor, new AddConfigurationSectionCommand(provider, typeof(DatabaseDistributorNode), DatabaseDistributorSettings.SectionName), ServiceHelper.GetCurrentRootNode(provider), Shortcut.None, SR.DatabaseDistributorNode, InsertionPoint.New);

			// Get the UI Hierarchy for the currently selected application configuration node
			IUIHierarchy currentHierarchy = ServiceHelper.GetCurrentHierarchy(provider);

			// Search down through the hierachy to see if the application already contains the
			// Database Distributor settings node. Menu item will not be enabled if the application is 
			// already configured to use the block.
			bool containsNode = currentHierarchy.ContainsNodeType(typeof(DatabaseDistributorNode));
			item.Enabled = !containsNode;

			// Retrieve the service that handles menus, and use it to add our menu item.
			IMenuContainerService menuService = provider.GetService(typeof(IMenuContainerService)) as IMenuContainerService;
			menuService.MenuItems.Add(item);
		}

		private static void RegisterNodeMaps(IServiceProvider serviceProvider)
		{
			// Retrieve the INodeCreationService, which associatees node types with runtime configuration data types
			INodeCreationService nodeCreationService = serviceProvider.GetService(typeof(INodeCreationService)) as INodeCreationService;

			// Associate the nodes for providers with their respective runtime configuration data types. Once associated, provider nodes can be
			// created from their runtime configuration data types.
			Type nodeType = typeof(DatabaseDistributorNode);
			NodeCreationEntry entry = NodeCreationEntry.CreateNodeCreationEntryWithMultiples(new AddChildNodeCommand(serviceProvider, nodeType), nodeType, typeof(DatabaseDistributorSettings), SR.DatabaseDistributorNode);
			nodeCreationService.AddNodeCreationEntry(entry);
		}
 

		/// <summary>
		/// Reads the configuration settings from storage and returns a node representing the settings.
		/// </summary>
		public void Open(IServiceProvider serviceProvider)
		{
			ConfigurationContext configurationContext = ServiceHelper.GetCurrentConfigurationContext(serviceProvider);

			// See of the application configuration includes the Database Distributor. If the
			// configuration does not include a configuration section for the block, then
			// the Open method returns without performing any action.  
			if (configurationContext.IsValidSection(DatabaseDistributorSettings.SectionName))
			{
				DatabaseDistributorSettings databaseDistributorSettings = null;
				DatabaseDistributorNode databaseDistributorNode = null;
				try
				{
					// Read the settings and construct the configuration node 
					databaseDistributorSettings = (DatabaseDistributorSettings)configurationContext.GetConfiguration(DatabaseDistributorSettings.SectionName);
					databaseDistributorNode = new DatabaseDistributorNode(databaseDistributorSettings);
				}
				catch (ConfigurationException e)
				{
					ServiceHelper.LogError(serviceProvider, databaseDistributorNode, e);
				}

				// Retrieve the application configuration node, and add
				// the Database Distributor configuration node as a child configuration node.
				ConfigurationNode configurationNode = ServiceHelper.GetCurrentRootNode(serviceProvider);
				configurationNode.Nodes.Add(databaseDistributorNode);
			}
		}

		/// <summary>
		/// Save the Database Distributor settings to the application configuration.
		/// </summary>
		public void Save(IServiceProvider serviceProvider)
		{
			ConfigurationContext builder = ServiceHelper.GetCurrentConfigurationContext(serviceProvider);

			// Verify that the application has been configured to use the Database Distributor
			if (builder.IsValidSection(DatabaseDistributorSettings.SectionName))
			{
				DatabaseDistributorNode databaseDistributorNode = null;
				try
				{
					// Attempt to retrieve the node that has the Data Mapping settings
					IUIHierarchy hierarchy = ServiceHelper.GetCurrentHierarchy(serviceProvider);
					databaseDistributorNode = hierarchy.FindNodeByType(typeof(DatabaseDistributorNode)) as DatabaseDistributorNode;

					// If application isn't configured to use the block the node won't be found. 
					if (databaseDistributorNode == null)
					{
						return;
					}

					// Save the settings to storage
					DatabaseDistributorSettings databaseDistributorSettings = databaseDistributorNode.DatabaseDistributorSettings;
					builder.WriteConfiguration(DatabaseDistributorSettings.SectionName, databaseDistributorSettings);
				}
				catch (ConfigurationException e)
				{
					ServiceHelper.LogError(serviceProvider, databaseDistributorNode, e);
				}
				catch (InvalidOperationException e)
				{
					ServiceHelper.LogError(serviceProvider, databaseDistributorNode, e);
				}
			}
		}

		/// <summary>
		/// Adds to the dictionary configuration data for the Data configuration section.
		/// </summary>
		public void BuildContext(IServiceProvider serviceProvider, ConfigurationDictionary configurationDictionary)
		{
			IUIHierarchy hierarchy = ServiceHelper.GetCurrentHierarchy(serviceProvider);
			if (hierarchy == null) return;

			DatabaseDistributorNode databaseDistributorNode = hierarchy.FindNodeByType(typeof(DatabaseDistributorNode)) as DatabaseDistributorNode;
			if (databaseDistributorNode == null)
			{
				return;
			}
			configurationDictionary[DatabaseDistributorSettings.SectionName] = databaseDistributorNode.DatabaseDistributorSettings;
		}
	}
}
