using System;
using System.Data;
using System.Drawing.Design;
using System.Windows.Forms;
using System.ComponentModel;
using System.Xml.Serialization;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Logging.Distributor;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.Design;
using DatabaseDistributor.Configuration;

namespace DatabaseDistributor.Design
{
    /// <summary>
    /// Node representing a <see cref="DatabaseDistributor"/>.
    /// </summary>
	[ServiceDependency(typeof(ILinkNodeService))]
	[ServiceDependency(typeof(IXmlIncludeTypeService))]
	[ServiceDependency(typeof(INodeCreationService))]
	[Image(typeof(DatabaseDistributorNode))]
	public class DatabaseDistributorNode : ConfigurationNode
    {
		private InstanceNode instanceNode;
		private ConfigurationNodeChangedEventHandler onInstanceNodeRemoved;
		private ConfigurationNodeChangedEventHandler onInstanceNodeRenamed;
		private DatabaseDistributorSettings databaseDistributorSettings;
		private const string defaultStoredProc = "ReadLogEntries";

        /// <summary>
        /// Creates node with initial configuration data.
        /// </summary>
        public DatabaseDistributorNode() : this(new DatabaseDistributorSettings(SR.DatabaseDistributor,defaultStoredProc))
        {
        }

        /// <summary>
        /// Creates node with specified configuration data.
        /// </summary>
        /// <param name="databaseDistributionStrategyData">Configuration data.</param>
        public DatabaseDistributorNode(DatabaseDistributorSettings databaseDistributorSettings) : base()
        {
			if (databaseDistributorSettings == null)
			{
				throw new ArgumentNullException("databaseDistributorSettings");
			}

			this.onInstanceNodeRemoved += new ConfigurationNodeChangedEventHandler(OnInstanceNodeRemoved);
			this.onInstanceNodeRenamed += new ConfigurationNodeChangedEventHandler(OnInstanceNodeRenamed);
			this.databaseDistributorSettings = databaseDistributorSettings;
        }

		public DatabaseDistributorSettings DatabaseDistributorSettings
		{
			get
			{
				return this.databaseDistributorSettings;
			}
		}

		/// <summary>
		/// The configured name.
		/// </summary>
		[ReadOnly(true)]
		[Browsable(false)]
		public override string Name
		{
			get { return base.Name; }
			set { base.Name = value; }
		}

		/// <summary>
		/// <para>After sited, sets the name of the node.</para>
		/// </summary>
		protected override void OnSited()
		{
			base.OnSited ();
			Site.Name = SR.DatabaseDistributorNode;
		}
		
		/// <summary>
		/// Name of the database distributor windows service.
		/// </summary>
		[Required]
		[SRDescription(SR.Keys.DistributorServiceNameDescription)]
		[SRCategory(SR.Keys.CategoryGeneral)]
		public string ServiceName
		{
			get { return this.databaseDistributorSettings.ServiceName; }
			set { this.databaseDistributorSettings.ServiceName = value; }
		}
		
		/// <summary>
		/// Timer interval to poll the database in the Database distributor windows service.
		/// </summary>
		[Required]
		[SRDescription(SR.Keys.PollingIntervalDescription)]
		[SRCategory(SR.Keys.CategoryGeneral)]
		public int PollingInterval
		{
			get { return this.databaseDistributorSettings.PollingInterval; }
			set { this.databaseDistributorSettings.PollingInterval = value; }
		}

		/// <summary>
        /// Stored Proc for Writing Log Entry.
        /// </summary>
        [Required]
        [SRDescription(SR.Keys.StoredProcDescription)]
        [SRCategory(SR.Keys.CategoryGeneral)]
        public string ReadStoredProcedure
        {
            get { return databaseDistributorSettings.StoredProcName; }
            set { databaseDistributorSettings.StoredProcName = value; }
        }

		[Editor(typeof(ReferenceEditor), typeof(UITypeEditor))]
		[ReferenceType(typeof(InstanceNode))]    
		[SRCategory(SR.Keys.CategoryGeneral)]
		public InstanceNode DatabaseInstance
		{
			get { return instanceNode; }
			set
			{
				ILinkNodeService service = GetService(typeof(ILinkNodeService)) as ILinkNodeService;
				instanceNode = service.CreateReference(instanceNode, value, onInstanceNodeRemoved, onInstanceNodeRenamed) as InstanceNode;
				databaseDistributorSettings.DatabaseInstanceName = string.Empty;
				if (instanceNode != null)
				{
					databaseDistributorSettings.DatabaseInstanceName = instanceNode.Name;	
				}
			}
		}

//        /// <summary>
//        /// Read only.  Returns the type name of a <see cref="DatabaseDistributionStrategy.DatabaseDistributionStrategy"/>.
//        /// </summary>
//        [Browsable(false)]
//        public override string TypeName
//        {
//            get { return typeof(DatabaseDistributor).AssemblyQualifiedName; }
//        }
//
//        /// <summary>
//        /// Returns the <c>TypeName</c> for visibility in the UI.
//        /// </summary>
//        [ReadOnly(true)]
//        [SRDescription(SR.Keys.DistributionStrategyTypeNameDescription)]
//        [SRCategory(SR.Keys.CategoryGeneral)]
//        public string Type
//        {
//            get { return TypeName; }
//        }

		/// <summary>
		/// See <see cref="ConfigurationNode.ResolveNodeReferences"/>.
		/// </summary>
		public override void ResolveNodeReferences()
		{
			base.ResolveNodeReferences();
			InstanceCollectionNode instanceCollectionNode = Hierarchy.FindNodeByType(typeof(InstanceCollectionNode)) as InstanceCollectionNode;
			DatabaseInstance = Hierarchy.FindNodeByName(instanceCollectionNode, databaseDistributorSettings.DatabaseInstanceName) as InstanceNode;
		}

		private void OnInstanceNodeRemoved(object sender, ConfigurationNodeChangedEventArgs e)
		{
			this.instanceNode = null;
		}

		private void OnInstanceNodeRenamed(object sender, ConfigurationNodeChangedEventArgs e)
		{
			databaseDistributorSettings.DatabaseInstanceName = e.Node.Name;
		}
    }
}