using System;
using System.Globalization;
using System.Threading;
using System.Timers;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Distributor;
using Timer = System.Timers.Timer;

namespace DatabaseDistributor
{
    /// <summary>
    /// Represents the database polling timer.  Uses an <see cref="DatabaseDistributor"/> 
    /// to check for new log messages each timer interval.
    /// </summary>
    internal class DatabaseListener
    {
        internal int ListenerRetries = 30;

        private const int DefaultTimerInterval = 20000;
        private const int ListenerRetrySleepTime = 1000;
        private static int timerInterval;

        private System.Timers.Timer databaseTimer = null;
        private DistributorEventLogger eventLogger = null;

        private DatabaseDistributorService distributorService;
        private DatabaseLogDistributor logDistributor;

        /// <summary>
        /// Initialize a new instance of the <see cref="MsmqListener"/>.
        /// </summary>
        /// <param name="distributorService">Distributor service inheriting from <see cref="System.ServiceProcess.ServiceBase"/>.</param>
        /// <param name="timerInterval">Interval to check for new messages.</param>
        public DatabaseListener(DatabaseDistributorService distributorService, int timerInterval)
        {
            this.distributorService = distributorService;
            this.DatabaseTimerInterval = timerInterval;
            this.eventLogger = distributorService.EventLogger;

            this.logDistributor = new DatabaseLogDistributor(ConfigurationManager.GetCurrentContext(), this.eventLogger);
        }

        /// <summary>
        /// Polling interval to check for new log messages.
        /// </summary>
        public int DatabaseTimerInterval
        {
            get
            {
                if (timerInterval == 0)
                {
                    return DefaultTimerInterval;
                }
                else
                {
                    return timerInterval;
                }
            }

            set
            {
                if (value == 0)
                {
                    timerInterval = DefaultTimerInterval;
                }
                else
                {
                    timerInterval = value;
                }
            }
        }

        /// <summary>
        /// Start the listener and begin polling the database.
        /// </summary>
        public virtual void StartListener()
        {
            try
            {
                this.eventLogger.AddMessage(SR.ListenerStartingMessage, SR.ListenerStarting);

                this.logDistributor.StopReceiving = false;

                if (this.databaseTimer == null)
                {
                    this.databaseTimer = new Timer();
                    this.databaseTimer.Elapsed += new ElapsedEventHandler(OnDatabaseTimedEvent);
                }
                this.databaseTimer.Interval = this.DatabaseTimerInterval;
                this.databaseTimer.Enabled = true;

                this.eventLogger.AddMessage(SR.ListenerStartCompleteMessage, SR.ListenerStartComplete(this.DatabaseTimerInterval));
            }
            catch (Exception e)
            {
                this.eventLogger.AddMessage(SR.ListenerStartErrorMessage, SR.ListenerStartError);
                this.eventLogger.AddMessage("exception", e.Message);
                throw;
            }
            catch
            {
                this.eventLogger.AddMessage(SR.ListenerStartErrorMessage, SR.ListenerStartError);
                this.eventLogger.AddMessage("exception", SR.UnknownError);
                throw;
            }
        }

        /// <summary>
        /// Attempt to stop the database listener and discontinue polling the database.
        /// </summary>
        /// <returns>True if the listener stopped succesfully.</returns>
        public virtual bool StopListener()
        {
            try
            {
                if (this.databaseTimer != null)
                {
                    this.eventLogger.AddMessage(SR.ListenerStopStartedMessage, SR.ListenerStopStarted);

                    this.databaseTimer.Enabled = false;
                    this.logDistributor.StopReceiving = true;

                    if (WaitUntilListenerStopped())
                    {
                        return true;
                    }

                    this.databaseTimer.Enabled = true;
                    this.logDistributor.StopReceiving = false;
                    return false;
                }
                return true;
            }
            catch (Exception e)
            {
                this.eventLogger.AddMessage(SR.ListenerStopErrorMessage, SR.ListenerStopError);
                this.eventLogger.AddMessage("exception", e.Message);
                throw;
            }
            catch
            {
                this.eventLogger.AddMessage(SR.ListenerStopErrorMessage, SR.ListenerStopError);
                this.eventLogger.AddMessage("exception", SR.UnknownError);
                throw;
            }
        }

        private bool WaitUntilListenerStopped()
        {
            int timeOut = 0;
            while (timeOut < ListenerRetries)
            {
                // Try to stop for DatabaseListenerRetries retries (1 second per retry)
                if (this.logDistributor.IsCompleted)
                {
                    this.eventLogger.AddMessage(SR.ListenerStopCompletedMessage,
                                                SR.ListenerStopCompleted(timeOut.ToString(CultureInfo.InvariantCulture)));

                    return true;
                }
                Thread.Sleep(ListenerRetrySleepTime);
                ++timeOut;
            }

            this.eventLogger.AddMessage(SR.StopListenerWarningMessage,
                                        SR.ListenerCannotStop(timeOut.ToString(CultureInfo.InvariantCulture)));

            return false;
        }

        /// <devdoc>
        /// support unit tests - allows for a mock object
        /// </devdoc>
        internal void SetDatabaseLogDistributor(DatabaseLogDistributor logDistributor)
        {
            this.logDistributor = logDistributor;
        }

        /// <devdoc>
        /// Event triggered by the database timer event handler. 
        /// This method runs regularly to check the database for pending logentries.
        /// </devdoc>
        private void OnDatabaseTimedEvent(object sender, ElapsedEventArgs e)
        {
            try
            {
                if (this.logDistributor.IsCompleted &&
                    (this.distributorService.Status == ServiceStatus.OK))
                {
                    this.logDistributor.CheckForMessages();
                }
            }
            catch (Exception err)
            {
                this.eventLogger.AddMessage("HEADER", SR.DatabaseTimedEventError);
                this.eventLogger.WriteToLog(err, Severity.Error);
                this.distributorService.Status = ServiceStatus.Shutdown;
            }
            catch
            {
                this.distributorService.Status = ServiceStatus.Shutdown;
            }
        }
    }
}