using System;
using System.Data;
using System.IO;
using System.Messaging;
using System.Runtime.Serialization.Formatters.Soap;
using System.Security.Principal;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Distributor;
using Microsoft.Practices.EnterpriseLibrary.Logging.Instrumentation;
using DatabaseDistributor.Configuration;

namespace DatabaseDistributor
{
    /// <summary>
    /// Receive new log messages from the databse and distribute each log entry.
    /// </summary>
    internal class DatabaseLogDistributor : LogDistributor
    {
        private const string Header = "HEADER";

        private bool isCompleted = true;
        private bool stopReceiving = false;
		private readonly DatabaseDistributorConfigurationView dbDistributorConfigurationView;

        private DistributorEventLogger eventLogger;

		/// <summary>
		/// Deserialize the LogEntry 
		/// </summary>
		/// <param name="logEntryString">The serialized log entry</param>
		/// <returns>A LogEntry that has been deserialized.</returns>
		protected LogEntry DeserializeEntry(string logEntryString)
		{
			byte[] buffer = Encoding.UTF8.GetBytes(logEntryString);
			LogEntry logEntry;
			using (MemoryStream ms = new MemoryStream(buffer))
			{
				SoapFormatter formatter = new SoapFormatter();
				logEntry = (LogEntry)formatter.Deserialize(ms);
			}
			return logEntry;
		}
		
		/// <summary>
        /// Setup the database and the formatter of the messages.
        /// </summary>
        public DatabaseLogDistributor(ConfigurationContext configurationContext) : this(configurationContext, new DistributorEventLogger())
        {
        }

        /// <summary>
        /// Set up the database with the specified logger
        /// </summary>
        /// <param name="configurationContext"><see cref="ConfigurationContext"></see> containing current configuration information</param>
        /// <param name="eventLogger"><see cref="DistributorEventLogger"></see> to use for distributing events from this object</param>
        public DatabaseLogDistributor(ConfigurationContext configurationContext, DistributorEventLogger eventLogger) : base(configurationContext)
        {
			dbDistributorConfigurationView = new DatabaseDistributorConfigurationView(configurationContext);
			this.eventLogger = eventLogger;
        }

        /// <summary>
        /// Read-only property to check if the synchronous receive is completed.
        /// </summary>
        public virtual bool IsCompleted
        {
            get { return this.isCompleted; }
        }

        /// <summary>
        /// Instructs the listener to stop receiving messages.
        /// </summary>
        public virtual bool StopReceiving
        {
            get { return this.stopReceiving; }
            set { this.stopReceiving = value; }
        }

        /// <summary>
        /// Start receiving the message(s) from the database.
        /// The messages will be taken from the database until no more are left to process.
        /// This method is triggered every x seconds. (x is defined in application configuration file)
        /// </summary>
        public virtual void CheckForMessages()
        {
            try
            {
                ReceiveMessages();
            }
           catch (LoggingException)
            {
                throw;
            }
            catch (Exception ex)
            {
				DatabaseDistributorSettings dbDistribSettings = dbDistributorConfigurationView.GetDatabaseDistributorSettings();
				string database = dbDistribSettings.DatabaseInstanceName;
				string storedProc = dbDistribSettings.StoredProcName;
                string errorMsg = SR.DatabaseReceiveGeneralError(database,storedProc);
                this.eventLogger.AddMessage(Header, errorMsg);
                this.eventLogger.WriteToLog(ex, Severity.Error);

                throw new LoggingException(errorMsg, ex);
            }
            catch
            {
				DatabaseDistributorSettings dbDistribSettings = dbDistributorConfigurationView.GetDatabaseDistributorSettings();
				string database = dbDistribSettings.DatabaseInstanceName;
				string storedProc = dbDistribSettings.StoredProcName;
				string errorMsg = SR.DatabaseReceiveGeneralError(database,storedProc);
				this.eventLogger.AddMessage(Header, errorMsg);

                throw new LoggingException(errorMsg);
            }
            finally
            {
                this.isCompleted = true;
            }
        }

        private void ReceiveMessages()
        {
            this.isCompleted = false;

			IDataReader dataReader = GetDataReader();
			if (dataReader != null)
			{
				while (dataReader.Read() && !(this.isCompleted))
				{
					string serializedEntry = dataReader.GetString(0);
					LoggingLogDistributedEvent.Fire(SR.DatabaseLogRead(serializedEntry));
					if (serializedEntry.IndexOf("<SOAP-ENV:Envelope") > -1)
					{
						LogEntry logEntry = DeserializeEntry(serializedEntry);
						if (logEntry != null)
						{
							this.ProcessLog(logEntry);

							//TODO: call MarkLogEntryAsReadSproc
						}
					}

					if (this.StopReceiving)
					{
						this.isCompleted = true;
					}
				} //end while

				dataReader.Close();
			}

        }

        private IDataReader GetDataReader()
        {
			DatabaseDistributorSettings dbDistribSettings = dbDistributorConfigurationView.GetDatabaseDistributorSettings();
			
			Database db = null;
			DatabaseProviderFactory factory = new DatabaseProviderFactory(dbDistributorConfigurationView.ConfigurationContext);
			db = ((dbDistribSettings.DatabaseInstanceName == null) || (dbDistribSettings.DatabaseInstanceName.Equals(String.Empty)))? factory.CreateDefaultDatabase() 
					: factory.CreateDatabase(dbDistribSettings.DatabaseInstanceName);

			IDataReader dataReader = db.ExecuteReader(System.Data.CommandType.StoredProcedure,dbDistribSettings.StoredProcName);
			return dataReader;
        }
    }
}