using System;
using System.Collections;
using System.Globalization;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Distributor.Configuration;

namespace Microsoft.Practices.EnterpriseLibrary.Logging.Formatters
{
    /// <summary>
    /// Represents a template based formatter for <see cref="LogEntry"/> messages.
    /// </summary>
    public class TextFormatterWithELDictionary : TextFormatter	//ConfigurationProvider, ILogFormatter
    {
        private ELTextFormatterData customFormatterData;

        /// <summary>
        /// Message template containing tokens.
        /// </summary>
        private StringBuilder templateBuilder;

        /// <summary>
        /// Array of token formatters.
        /// </summary>
        private ArrayList tokenFunctions;

        private const string timeStampToken = "{timestamp}";
        private const string messageToken = "{message}";
        private const string categoryToken = "{category}";
        private const string priorityToken = "{priority}";
        private const string eventIdToken = "{eventid}";
        private const string severityToken = "{severity}";
        private const string titleToken = "{title}";

        private const string machineToken = "{machine}";
        private const string appDomainNameToken = "{appDomain}";
        private const string processIdToken = "{processId}";
        private const string processNameToken = "{processName}";
        private const string threadNameToken = "{threadName}";
        private const string win32ThreadIdToken = "{win32ThreadId}";

        private const string NewLineToken = "{newline}";
        private const string TabToken = "{tab}";

        /// <summary>
        /// Internal only for testing purposes
        /// </summary>
//        internal StringBuilder TemplateBuilder
//        {
//            get { return templateBuilder; }
//            set { templateBuilder = value; }
//        }

        /// <summary>
        /// Initializes a new instance of a <see cref="customFormatterWithELDictionary"></see>
        /// </summary>
        public TextFormatterWithELDictionary() : this(new ELTextFormatterData())
        {
        }

        /// <summary>
        /// Initializes a new instance of a <see cref="customFormatter"></see> with the given 
        /// <see cref="customFormatterData"></see> configuration data
        /// </summary>
        /// <param name="customFormatterData">
        /// <para>A <see cref="customFormatterData"/> object.</para>
        /// </param>
        public TextFormatterWithELDictionary(ELTextFormatterData customFormatterData)
        {
            ArgumentValidation.CheckForNullReference(customFormatterData, "customFormatterData");

            this.customFormatterData = customFormatterData;
            RegisterTemplate();
        }

        /// <summary>
        /// Initialize a new instance of a <see cref="customFormatter"/>.  Reads the template from
        /// configuration parameters.
        /// </summary>
        /// <param name="configurationView">Dynamic view into the configuration information needed by the <see cref="customFormatter"></see></param>     
        public override void Initialize(ConfigurationView configurationView)
        {
            ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
            ArgumentValidation.CheckExpectedType(configurationView, typeof(LoggingConfigurationView));

            LoggingConfigurationView loggingConfigurationView = (LoggingConfigurationView)configurationView;
            FormatterData formatterData = loggingConfigurationView.GetFormatterData(ConfigurationName);
            ArgumentValidation.CheckExpectedType(customFormatterData, typeof(ELTextFormatterData));

            customFormatterData = (ELTextFormatterData)formatterData;
            RegisterTemplate();
        }

        private void RegisterTemplate()
        {
            this.templateBuilder = new StringBuilder(customFormatterData.Template.Value);
            RegisterTokenFunctions();
        }

        /// <summary>
        /// Gets the <see cref="customFormatterData"/> from which
        /// this object was initialized.
        /// </summary>
        protected new ELTextFormatterData FormatterData
        {
            get { return customFormatterData; }
        }

        /// <summary>
        /// Format the <see cref="LogEntry"/> object by replacing tokens with values.
        /// </summary>
        /// <param name="log">Log entry to format.</param>
        /// <returns>Formatted string with tokens replaced with property values.</returns>
        public override string Format(LogEntry log)
        {
            if (templateBuilder.Length == 0)
            {
                templateBuilder = new StringBuilder(SR.DefaultTextFormat);
            }

            templateBuilder.Replace(timeStampToken, log.TimeStampString);
            templateBuilder.Replace(titleToken, log.Title);
            templateBuilder.Replace(messageToken, log.Message);
            templateBuilder.Replace(categoryToken, log.Category);
            templateBuilder.Replace(eventIdToken, log.EventId.ToString(CultureInfo.InvariantCulture));
            templateBuilder.Replace(priorityToken, log.Priority.ToString(CultureInfo.InvariantCulture));
            templateBuilder.Replace(severityToken, log.Severity.ToString(CultureInfo.InvariantCulture));

            templateBuilder.Replace(machineToken, log.MachineName);
            templateBuilder.Replace(appDomainNameToken, log.AppDomainName);
            templateBuilder.Replace(processIdToken, log.ProcessId);
            templateBuilder.Replace(processNameToken, log.ProcessName);
            templateBuilder.Replace(threadNameToken, log.ManagedThreadName);
            templateBuilder.Replace(win32ThreadIdToken, log.Win32ThreadId);

            FormatTokenFunctions(log);

            templateBuilder.Replace(NewLineToken, Environment.NewLine);
            templateBuilder.Replace(TabToken, "\t");

            return templateBuilder.ToString();
        }

        private void FormatTokenFunctions(LogEntry log)
        {
            foreach (TokenFunction token in tokenFunctions)
            {
                token.Format(templateBuilder, log);
            }
        }

        private void RegisterTokenFunctions()
        {
            tokenFunctions = new ArrayList();
            tokenFunctions.Add(new ELDictionaryToken());
            tokenFunctions.Add(new KeyValueToken());
            tokenFunctions.Add(new TimeStampToken());
        }
    }
}