using System;

using System.Globalization;
using System.Security.Principal;

using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Instrumentation;
using System.DirectoryServices;
using Authentication.Providers.ADAuthenticationProvider.Configuration;


namespace Authentication.Providers.ADAuthenticationProvider
{
	
	/// <summary>
	/// Defines the functionality for an Active Directory-based Authentication Provider.
	/// </summary>
	public class ADAuthenticationProvider : ConfigurationProvider, IAuthenticationProvider
	{
		private SecurityConfigurationView securityConfigurationView;

		/// <summary>
		/// <para>Initialize a new instance of the <see cref="DbAuthenticationProvider"/> class.</para>
		/// </summary>
		public ADAuthenticationProvider()
		{}

		
		/// <summary>
		/// <para>Initializes the provider with a <see cref="SecurityConfigurationView"/>.</para>
		/// </summary>
		/// <param name="configurationView">
		/// <para>A <see cref="SecurityConfigurationView"/> object.</para>
		/// </param>
		public override void Initialize(ConfigurationView configurationView)
		{
			ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
			ArgumentValidation.CheckExpectedType(configurationView, typeof(SecurityConfigurationView));

			this.securityConfigurationView = (SecurityConfigurationView)configurationView;
		}

		/// <summary>
		/// Authenticate the credentials against Active Directory.
		/// </summary>
		/// <param name="credentials">A credential object such as a <see cref="NamePasswordCredential"/>.</param>
		/// <param name="userIdentity">An <see cref="IIdentity"/> object representing 
		/// authenticated credentials returned if successfull.</param>
		/// <returns><strong>True</strong> if authentication was
		/// successful, otherwise false.</returns>
		/// <remarks>
		/// This method call is intended to be overloaded to support additional 
		/// credential objects if/when they are brought online.
		/// </remarks>
		public bool Authenticate(object credentials, out IIdentity userIdentity)
		{
			bool result = false;
			userIdentity = null;

			NamePasswordCredential namePasswordCredentials = credentials as NamePasswordCredential;

			if (namePasswordCredentials != null && namePasswordCredentials.Name.Length > 0)
			{
				SecurityAuthenticationCheckEvent.Fire(namePasswordCredentials.Name);

				result = this.Login(namePasswordCredentials.Name, namePasswordCredentials.Password);
				
				if (result)
				{
					userIdentity = new GenericIdentity(namePasswordCredentials.Name, GetAuthenticationType());
				}
				else
				{
					SecurityAuthenticationFailedEvent.Fire(namePasswordCredentials.Name);
				}
			}

			return result;
		}

		//*********************************************************************
		//
		// ADUser.Login() Method
		//
		// The xyz method  attempts to bind to Active Directory with the \
		// UserName and password as input parameters.
		// If bind is successful, method returns true.
		//
		//*********************************************************************

		private bool Login(string userName, string password)
		{
			bool result = false;
			
			//Bind to rootDSE to get the Domain Name
			DirectoryEntry rootDSE = new DirectoryEntry("LDAP://RootDSE");
			
			//couldn't bind to the directory
			if ( rootDSE == null)
				return false;

			string domainName = Convert.ToString(rootDSE.Properties["defaultNamingContext"].Value);
			//rootDSE.Dispose();

			//Bind to the domain root
			DirectoryEntry domainRoot = new DirectoryEntry("LDAP://" + domainName);
			string path = domainRoot.Path;

			//Create the searcher, bound to the domain root
			DirectorySearcher dirSearch = new DirectorySearcher(domainRoot);
			
			//Retrieve the 'cn' along with the adspath, which is returned by default
			dirSearch.PropertiesToLoad.Add("cn");

			//dirSearch.Filter = String.Format("(& (SAMAccountName={0})(objectCategory=person)(objectClass=user) )",userName);
			dirSearch.Filter = String.Format("(& (SAMAccountName={0}) )",userName);
			SearchResult searchResult = dirSearch.FindOne();
			if (searchResult == null)
			{
				return false;
			}
//			domainRoot.Dispose();
//			dirSearch.Dispose();

			string adsPathToUser = Convert.ToString(searchResult.Properties["adspath"][0]);
			DirectoryEntry userEntry;

			//'userName' is the sAMaccountName
			userEntry = new DirectoryEntry(adsPathToUser, userName, password);
			if (userEntry != null)
			{
				try
				{
					string commonName = Convert.ToString(userEntry.Properties["cn"].Value);
					if ( (commonName != null) && (! commonName.Equals(String.Empty)))
					{
						result = true;
					}
				}
				catch (Exception ex)
				{
					string thisError = ex.ToString();
					result = false;
				}
//				finally
//				{
//					userEntry.Dispose();
//				}
			}

			return result;
		}

		private ADAuthenticationProviderData GetADAuthenticationProviderData()
		{
			AuthenticationProviderData authProviderData = securityConfigurationView.GetAuthenticationProviderData(ConfigurationName);
			ArgumentValidation.CheckExpectedType(authProviderData, typeof (ADAuthenticationProviderData));
    
			ADAuthenticationProviderData adAuthProviderData = (ADAuthenticationProviderData) authProviderData;
			return adAuthProviderData;
		}

		private string GetAuthenticationType()
		{
			ADAuthenticationProviderData adAuthenticationProviderData = (ADAuthenticationProviderData)securityConfigurationView.GetAuthenticationProviderData(ConfigurationName);
			return string.Format(CultureInfo.InvariantCulture, "{0}.{1}", typeof(ADAuthenticationProvider).FullName, adAuthenticationProviderData.Name);
		}
	}
}
