using System;
using System.Globalization;
using System.Security;
using System.Security.Principal;
using System.Security.Permissions;
using System.Threading;

using Microsoft.Practices.EnterpriseLibrary.Security;

namespace EAF.Security.AuthorizationProviderPermissions
{
	[Serializable()]
	internal class AuthorizationProviderInfo
	{
		#region "Fields"
			internal bool authenticated;
			internal string providerName;
			internal string rule;
		#endregion

		#region "Methods"
			public AuthorizationProviderInfo() {}

			internal void FromXml(SecurityElement e)
			{
				string text1 = e.Attribute("Authenticated");
				if (text1 != null)
				{
					this.authenticated = string.Compare(text1, "true", true, CultureInfo.InvariantCulture) == 0;
				}
				else
				{
					this.authenticated = false;
				}
				string text2 = e.Attribute("ProviderName");
				if (text2 != null)
				{
					this.providerName = text2;
				}
				else
				{
					this.providerName = null;
				}
				string text3 = e.Attribute("Rule");
				if (text3 != null)
				{
					this.rule = text3;
				}
				else
				{
					this.rule = null;
				}
			}
	 
			
			internal SecurityElement ToXml()
			{
				SecurityElement element1 = new SecurityElement("Identity");
				if (this.authenticated)
				{
					element1.AddAttribute("Authenticated", "true");
				}
				if (this.providerName != null)
				{
					element1.AddAttribute("ProviderName", SecurityElement.Escape(this.providerName));
				}
				if (this.rule != null)
				{
					element1.AddAttribute("Rule", SecurityElement.Escape(this.rule));
				}
				return element1;
			}
	 

		#endregion

	}

	/// <summary>
	/// Summary description for AuthorizationProviderPermission
	/// </summary>
	[Serializable()]
	public class AuthorizationProviderPermission : IPermission, IUnrestrictedPermission, ISecurityEncodable
	{
		// Methods
		public AuthorizationProviderPermission(PermissionState state)
		{
			if (state == PermissionState.Unrestricted)
			{
				this.arrProviderInfo = new AuthorizationProviderInfo[1] { new AuthorizationProviderInfo() } ;
				this.arrProviderInfo[0].authenticated = true;
				this.arrProviderInfo[0].providerName= null;
				this.arrProviderInfo[0].rule = null;
			}
			else
			{
				if (state != PermissionState.None)
				{
					throw new ArgumentException("Argument_InvalidPermissionState");
				}
				this.arrProviderInfo = new AuthorizationProviderInfo[1] { new AuthorizationProviderInfo() } ;
				this.arrProviderInfo[0].authenticated = false;
				this.arrProviderInfo[0].providerName = "";
				this.arrProviderInfo[0].rule = "";
			}
		}

		private AuthorizationProviderPermission(AuthorizationProviderInfo[] array) 
		{
			arrProviderInfo = array;
		}

		public AuthorizationProviderPermission(string rule) : this(null, rule, true)
		{
		}

		public AuthorizationProviderPermission(string providerName, string rule) : this(providerName, rule, true)
		{
		}
 
		public AuthorizationProviderPermission(string providerName, string rule, bool isAuthenticated)
		{
			this.arrProviderInfo = new AuthorizationProviderInfo[1] { new AuthorizationProviderInfo() } ;
			this.arrProviderInfo[0].authenticated = isAuthenticated;
			this.arrProviderInfo[0].providerName = providerName;
			this.arrProviderInfo[0].rule = rule;
		}
		
		public IPermission Copy() 
		{
			return new AuthorizationProviderPermission(this.arrProviderInfo);
		}

		public void Demand()
		{
			IPrincipal principal1 = Thread.CurrentPrincipal;
			if (principal1 == null)
			{
				throw new SecurityException("Security_AuthorizationProviderPermission", base.GetType(), this.ToXml().ToString());
			}
			if (this.arrProviderInfo != null)
			{
				int num1 = this.arrProviderInfo.Length;
				bool flag1 = false;
				for (int num2 = 0; num2 < num1; num2++)
				{
					if (this.arrProviderInfo[num2].authenticated)
					{
						IAuthorizationProvider authorizationProvider;
						if ((this.arrProviderInfo[num2].providerName == null) || (this.arrProviderInfo[num2].providerName.Equals(String.Empty)))
							authorizationProvider = AuthorizationFactory.GetAuthorizationProvider();
						else
							authorizationProvider = AuthorizationFactory.GetAuthorizationProvider(this.arrProviderInfo[num2].providerName);

						if ((this.arrProviderInfo[num2].rule != null) && !authorizationProvider.Authorize(principal1,this.arrProviderInfo[num2].rule))
						{
							throw new SecurityException("Security_AuthorizationProviderPermission", typeof(AuthorizationProviderPermission));
						}
						flag1 = true;
						break;
					}
					flag1 = true;
					break;
				}
				if (!flag1)
				{
					throw new SecurityException("Security_AuthorizationProviderPermission", typeof(AuthorizationProviderPermission));
				}
			}
		}
 

		public void FromXml(SecurityElement elem) 
		{
			//CodeAccessPermission.ValidateElement(elem, this);
			this.arrProviderInfo = null;
			if ((elem.Children != null) && (elem.Children.Count != 0))
			{
				int num1 = elem.Children.Count;
				int num2 = 0;
				this.arrProviderInfo = new AuthorizationProviderInfo[num1];
				System.Collections.IEnumerator enumerator1 = elem.Children.GetEnumerator();
				while (enumerator1.MoveNext())
				{
					AuthorizationProviderInfo rule1 = new AuthorizationProviderInfo();
					rule1.FromXml((SecurityElement) enumerator1.Current);
					this.arrProviderInfo[num2++] = rule1;
				}
			}
		}

//		internal static int GetTokenIndex();
		
		public IPermission Intersect(IPermission target) {return new AuthorizationProviderPermission(this.arrProviderInfo);}	//TODO: IMPLEMENT THIS
//		
		private bool IsEmpty()
		{
			for (int num1 = 0; num1 < this.arrProviderInfo.Length; num1++)
			{
				if (((this.arrProviderInfo[num1].providerName == null) || !this.arrProviderInfo[num1].providerName.Equals("")) || (((this.arrProviderInfo[num1].rule == null) || !this.arrProviderInfo[num1].rule.Equals("")) || this.arrProviderInfo[num1].authenticated))
				{
					return false;
				}
			}
			return true;
		}
 
		
		public bool IsSubsetOf(IPermission target) {return true;}									//TODO: IMPLEMENT THIS
		
		public bool IsUnrestricted()
		{
			for (int num1 = 0; num1 < this.arrProviderInfo.Length; num1++)
			{
				if (((this.arrProviderInfo[num1].providerName != null) || (this.arrProviderInfo[num1].rule != null)) || !this.arrProviderInfo[num1].authenticated)
				{
					return false;
				}
			}
			return true;
		}
 

//		int IBuiltInPermission.GetTokenIndex();
//		public override string ToString();
		
		public SecurityElement ToXml() 
		{
			SecurityElement element1 = new SecurityElement("Permission");
			Type type = typeof(AuthorizationProviderPermissionFacade);            
			element1.AddAttribute("class", String.Format("{0}, {1}", type.FullName, type.Assembly.GetName().Name));
			element1.AddAttribute("version", "1");
			element1.AddAttribute("unrestricted", false.ToString());
			int num1 = this.arrProviderInfo.Length;
			for (int num2 = 0; num2 < num1; num2++)
			{
				element1.AddChild(this.arrProviderInfo[num2].ToXml());
			}
			return element1;
		}

		public IPermission Union(IPermission other) {return new AuthorizationProviderPermission(this.arrProviderInfo);}	//TODO: IMPLEMENT THIS

//		private bool VerifyType(IPermission perm);

		// Fields
		private AuthorizationProviderInfo[] arrProviderInfo;
	}

	[Serializable()]
	internal class AuthorizationProviderPermissionFacade : AuthorizationProviderPermission
	{
		public AuthorizationProviderPermissionFacade(PermissionState state) : base(null)
		{
		}
	}
}
