using System;
using System.Collections.Specialized;
using System.Configuration;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Principal;
using Microsoft.Interop.Security.AzRoles;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Instrumentation;

using EAF.Security.AzManRoles.Configuration;

namespace EAF.Security.AzManRoles
{
    /// <summary>
    /// Represents the AzMan roles provider.  
    /// </summary>
    /// <remarks>
    /// Supports Windows authentication only.
    /// Requires AzMan on Windows Server 2003, or Windows XP users will need to install the 
    /// Windows Server 2003 Administration Pack.
    /// Implements the <see cref="IRolesProvider"/> interface.</remarks>
    public class AzManRolesProvider : SecurityRolesProvider
    {
        private SecurityConfigurationView securityConfigurationView;

        /// <summary>
        /// Read the configuration settings.
        /// </summary>
        /// <param name="configurationView">A <see cref="SecurityConfigurationView"></see> object</param>
        public override void Initialize(ConfigurationView configurationView)
        {
            ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
            ArgumentValidation.CheckExpectedType(configurationView, typeof (SecurityConfigurationView));

            securityConfigurationView = (SecurityConfigurationView) configurationView;
        }

		protected override string[] CollectAllUserRoles(IIdentity userIdentity)
		{
			AzManRolesProviderData azManRolesData = (AzManRolesProviderData)securityConfigurationView.GetRolesProviderData(ConfigurationName);

			string applicationName = azManRolesData.Application;
			try 
			{
				IAzApplication azApp = null;
				IAzClientContext clientCtx = GetClientContext(azManRolesData, userIdentity, applicationName, out azApp);            
				IAzScopes scopes = azApp.Scopes;            
				StringCollection rolesCollection = new StringCollection();
				
				// get the app roles
				Array varRoles = clientCtx.GetRoles(azManRolesData.Scope) as Array;
				Debug.Assert(varRoles != null);
				for(Int32 index =0; index < varRoles.Length; index++) 
				{
					rolesCollection.Add((String)varRoles.GetValue(index));
				}            
//				// get the scope roles
//				if (scopes != null) 
//				{
//					foreach(IAzScope scope in scopes) 
//					{
//						varRoles = (Array)clientCtx.GetRoles(scope.Name);
//						for(Int32 index =0; index < varRoles.Length; index++) 
//						{
//							rolesCollection.Add(String.Concat(scope.Name, @"\", (String)varRoles.GetValue(index)));
//						}            
//					}            
//				}            

				string[] arrRoles = new string[rolesCollection.Count];
				rolesCollection.CopyTo(arrRoles,0);
				return  arrRoles;
			} 
			catch (COMException comEx) 
			{
				throw new SecurityException(comEx.Message, comEx);
			}             
		}

		/// <devdoc>
        /// Gets the client context for the call based on the identity, system and parameters.
        /// </devdoc>        
        private IAzClientContext GetClientContext(AzManRolesProviderData data, IIdentity identity, String applicationName, out IAzApplication azApp)
        {
            WindowsIdentity winIdentity = identity as WindowsIdentity;
            if (winIdentity == null)
            {
                throw new ArgumentException(SR.WindowsIdentityOnly);
            }

            AzAuthorizationStoreClass store = new AzAuthorizationStoreClass();
            store.Initialize(0, data.StoreLocation, null);
            azApp = store.OpenApplication(applicationName, null);
            Debug.Assert(azApp != null, "could not open the application");

            ulong tokenHandle = (ulong) winIdentity.Token.ToInt64();
            IAzClientContext clientCtx = azApp.InitializeClientContextFromToken(tokenHandle, null);
            Debug.Assert(clientCtx != null, "could not get the context");
            return clientCtx;
        }

        private AzManRolesProviderData GetConfigurationData()
        {
            RolesProviderData rolesProviderData = securityConfigurationView.GetRolesProviderData(ConfigurationName);
            ArgumentValidation.CheckExpectedType(rolesProviderData, typeof (AzManRolesProviderData));
            return (AzManRolesProviderData) rolesProviderData;
        }
    
	}
}