using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing.Design;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration.Design;

namespace EAF.Security.Cache.HttpCachingStore.Configuration.Design
{
    /// <summary>
    /// Node representing configuration from <see cref="HttpCachingStoreProviderData"/>.
    /// </summary>
    public class HttpCachingStoreProviderNode : SecurityCacheProviderNode
    {
		private const int absoluteExpiration = 60;
		private const int slidingExpiration = 10;
		private HttpCachingStoreProviderData cachingStoreProviderData;
		private SymmetricAlgorithmProviderNode symmetricAlgorithmNode;
        private ConfigurationNodeChangedEventHandler onSymmetricAlgorithmNodeRemoved;
        private ConfigurationNodeChangedEventHandler onSymmetricAlgorithmNodeRenamed;

        /// <summary>
        /// <para>Initialize a new instance of the <see cref="HttpCachingStoreProviderNode"/> class.</para>
        /// </summary>
        public HttpCachingStoreProviderNode() : this(new HttpCachingStoreProviderData(SR.SecurityInstance, slidingExpiration, absoluteExpiration))
        {
        }

        /// <summary>
        /// <para>Initialzie a new instance of the <see cref="CachingStoreProviderNode"/> class with a <see cref="CachingStoreProviderData"/> instance.</para>
        /// </summary>
        /// <param name="data">
        /// <para>The <see cref="cachingStoreProviderData"/> instance to initialize the instance.</para>
        /// </param>
        public HttpCachingStoreProviderNode(HttpCachingStoreProviderData data) : base(data)
        {
            this.cachingStoreProviderData = data;
            this.onSymmetricAlgorithmNodeRemoved = new ConfigurationNodeChangedEventHandler(OnSymmetricAlgorithmNodeRemoved);
            this.onSymmetricAlgorithmNodeRenamed = new ConfigurationNodeChangedEventHandler(OnSymmetricAlgorithmNodeRenamed);
        }

		/// <summary>
		/// See <see cref="CachingStoreProviderData.AbsoluteExpiration"/>.
		/// </summary>
		[Required]
		[AssertRange(0, RangeBoundaryType.Inclusive, Int32.MaxValue, RangeBoundaryType.Inclusive)]
		[SRDescription(SR.Keys.AbsoluteExpirationDescription)]
		[SRCategory(SR.Keys.CategoryGeneral)]
		public int AbsoluteExpiration
		{
			get { return cachingStoreProviderData.AbsoluteExpiration; }
			set { cachingStoreProviderData.AbsoluteExpiration = value; }
		}

		/// <summary>
		/// See <see cref="CachingStoreProviderData.SlidingExpiration"/>.
		/// </summary>
		[Required]
		[AssertRange(0, RangeBoundaryType.Inclusive, Int32.MaxValue, RangeBoundaryType.Inclusive)]
		[SRDescription(SR.Keys.SlidingExpirationDescription)]
		[SRCategory(SR.Keys.CategoryGeneral)]
		public int SlidingExpiration
		{
			get { return cachingStoreProviderData.SlidingExpiration; }
			set { cachingStoreProviderData.SlidingExpiration = value; }
		}
		
		/// <summary>
        /// See <see cref="CachingStoreProviderData.CacheManager"/>.
        /// </summary>
        [Required]
        [SRDescription(SR.Keys.SymmetricAlgorithmDescription)]
        [Editor(typeof(ReferenceEditor), typeof(UITypeEditor))]
        [ReferenceType(typeof(SymmetricAlgorithmProviderNode))]
        [SRCategory(SR.Keys.CategoryGeneral)]
        public SymmetricAlgorithmProviderNode SymmetricAlgorithm
        {
            get { return symmetricAlgorithmNode; }
            set
            {
                ILinkNodeService service = GetService(typeof(ILinkNodeService)) as ILinkNodeService;
                Debug.Assert(service != null, "Could not get the ILinkNodeService");
                this.symmetricAlgorithmNode = (SymmetricAlgorithmProviderNode)service.CreateReference(symmetricAlgorithmNode, value, onSymmetricAlgorithmNodeRemoved, onSymmetricAlgorithmNodeRenamed);
                cachingStoreProviderData.SymmetricAlgorithm = (symmetricAlgorithmNode == null) ? String.Empty : symmetricAlgorithmNode.Name;
            }
        }

        /// <summary>
        /// Returns the fully qualified assembly name of a <see cref="CachingStoreProvider"/>
        /// </summary>
        [Browsable(false)]
        public override string TypeName
        {
            get { return typeof(HttpCachingStoreProvider).AssemblyQualifiedName; }
        }

//        /// <summary>
//        /// See <see cref="ConfigurationNode.ResolveNodeReferences"/>.
//        /// </summary>
//        public override void ResolveNodeReferences()
//        {
//            base.ResolveNodeReferences();
//            CacheManagerCollectionNode categoryCollectionNode = Hierarchy.FindNodeByType(typeof(CacheManagerCollectionNode)) as CacheManagerCollectionNode;
//            Debug.Assert(categoryCollectionNode != null, "How is it that the cache managers are not there?");
//            if (categoryCollectionNode != null)
//            {
//                CacheManager = Hierarchy.FindNodeByName(categoryCollectionNode, this.cachingStoreProviderData.CacheManager) as CacheManagerNode;
//            }
//        }
//
//        /// <summary>
//        /// Initializes the Caching defaults.
//        /// </summary>
//        protected override void AddDefaultChildNodes()
//        {
//            base.AddDefaultChildNodes();
//            CreateCacheManagerSettingsNodeNode();
//            if (cachingStoreProviderData.CacheManager == null)
//            {
//                cachingStoreProviderData.CacheManager = SR.DefaultCacheManager;
//            }
//            ResolveNodeReferences();
//        }

        private void OnSymmetricAlgorithmNodeRemoved(object sender, ConfigurationNodeChangedEventArgs e)
        {
            this.symmetricAlgorithmNode = null;
        }

        private void OnSymmetricAlgorithmNodeRenamed(object sender, ConfigurationNodeChangedEventArgs e)
        {
            cachingStoreProviderData.SymmetricAlgorithm = e.Node.Name;
        }

//        private void CreateCacheManagerSettingsNodeNode()
//        {
//            if (!CacheManagerSettingsNodeExists())
//            {
//                AddConfigurationSectionCommand cmd = new AddConfigurationSectionCommand(Site, typeof(CacheManagerSettingsNode), CacheManagerSettings.SectionName);
//                cmd.Execute(Hierarchy.RootNode);
//            }
//        }
//
//        private bool CacheManagerSettingsNodeExists()
//        {
//            CacheManagerSettingsNode node = Hierarchy.FindNodeByType(typeof(CacheManagerSettingsNode)) as CacheManagerSettingsNode;
//            return (node != null);
//        }
    }
}