using System.Xml.Serialization;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;

namespace EAF.Security.Cache.HttpCachingStore.Configuration
{
    /// <summary>
    /// Configuration data for the Security Cache.
    /// </summary>
    [XmlRoot("httpCachingStoreProviderData", Namespace=SecuritySettings.ConfigurationNamespace)]
    public class HttpCachingStoreProviderData : SecurityCacheProviderData
    {
		private int slidingExpiration;
		private int absoluteExpiration;
		private string symmetricAlgorithm;

        /// <summary>
        /// <para>Initialize a new instance of the <see cref="EncryptedCachingStoreProviderData"/> class.</para>
        /// </summary>
        public HttpCachingStoreProviderData() : this(string.Empty)
        {
        }

        /// <summary>
        /// <para>Initialize a new instance of the <see cref="EncryptedCachingStoreProviderData"/> class with a name.</para>
        /// </summary>
        /// <param name="name">
        /// <para>The name of the provider.</para>
        /// </param>
        public HttpCachingStoreProviderData(string name) : this(name, 0, 0)
        {
        }

        /// <summary>
        /// <para>Initialize a new instance of the <see cref="CachingStoreProviderData"/> class with a name, sliding expiration and absolute expiration.</para>
        /// </summary>
        /// <param name="name">
        /// <para>The name of the provider.</para>
        /// </param>
        /// <param name="slidingExpiration">
        /// <para>The sliding expiration value.</para>
        /// </param>
        /// <param name="absoluteExpiration">
        /// <para>The absolute experation value.</para>
        /// </param>
        public HttpCachingStoreProviderData(string name, int slidingExpiration, int absoluteExpiration) : base(name)
        {
            this.SlidingExpiration = slidingExpiration;
            this.AbsoluteExpiration = absoluteExpiration;
        }

        /// <summary>
        /// <para>Initialize a new instance of the <see cref="CachingStoreProviderData"/> class with a name, sliding expiration, absolute expiration and <see cref="CacheManager"/> name.</para>
        /// </summary>
        /// <param name="name">
        /// <para>The name of the provider.</para>
        /// </param>
        /// <param name="slidingExpiration">
        /// <para>The sliding expiration value.</para>
        /// </param>
        /// <param name="absoluteExpiration">
        /// <para>The absolute experation value.</para>
        /// </param>
        /// <param name="cacheManager">
        /// <para>The name of the <see cref="CacheManager"/> to cache the information.</para>
        /// </param>
        public HttpCachingStoreProviderData(string name, int slidingExpiration, int absoluteExpiration, string symmetricAlgorithm) : 
			base(name)
        {
            this.symmetricAlgorithm = symmetricAlgorithm;
        }

        /// <summary>
        /// Gets or sets the Symmetric Algorithm instance name.
        /// </summary>
        /// <value>Symmetric Algorithm Name.</value>
        [XmlAttribute("symmetricAlgorithm")]
        public string SymmetricAlgorithm
        {
            get { return this.symmetricAlgorithm; }
            set { this.symmetricAlgorithm = value; }
        }

		/// <summary>
		/// Gets or sets the Sliding Session Expiration duration.
		/// </summary>
		/// <value>Sliding Session Expiration duration</value>
		[XmlAttribute(AttributeName = "defaultSlidingSessionExpirationInMinutes")]
		public int SlidingExpiration
		{
			get { return this.slidingExpiration; }
			set { this.slidingExpiration = value; }
		}

		/// <summary>
		/// Gets or sets the Absolute Session Expiration duration.
		/// </summary>
		/// <value>Absolute Session Expiration duration</value>
		[XmlAttribute(AttributeName = "defaultAbsoluteSessionExpirationInMinutes")]
		public int AbsoluteExpiration
		{
			get { return this.absoluteExpiration; }
			set { this.absoluteExpiration = value; }
		}

		/// <summary>
        /// <para>Gets or sets the <see cref="System.Type"/> name of the provider.</para>
        /// </summary>
        /// <value>
        /// <para>The type name of the provider. The default is an empty string.</para>
        /// </value>
        [XmlIgnore]
        public override string TypeName
        {
            get { return typeof(HttpCachingStoreProvider).AssemblyQualifiedName; }
            set
            {
            }
        }

    }
}