using System.ComponentModel;
using System.Diagnostics;
using System.Drawing.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using EAF.Security.PortalProfile.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration.Design;
using EnterpriseApplicationFramework.Data.Configuration;
using EnterpriseApplicationFramework.Data.Configuration.Design;

namespace EAF.Security.PortalProfile.Configuration.Design
{
    /// <summary>
    /// The design time representation for <see cref="DbProfileProviderData"/>.
    /// </summary>
    [ServiceDependency(typeof(ILinkNodeService))]
    public class PortalProfileProviderNode : ProfileProviderNode
    {
        private PortalProfileProviderData portalProfileProviderData;
        private DataSetMappingNode mapper;
        private ConfigurationNodeChangedEventHandler onDataMappingRemoved;
        private ConfigurationNodeChangedEventHandler onDataMappingRenamed;

        /// <summary>
        /// Initializes the node with default configuration data.
        /// </summary>
        public PortalProfileProviderNode() : this(new PortalProfileProviderData(SR.PortalProfileProviderDisplayName, string.Empty))
        {
        }

        /// <summary>
        /// Initializes the node with configuration data.
        /// </summary>
        /// <param name="dbProfileProviderData">The configuration data.</param>
        public PortalProfileProviderNode(PortalProfileProviderData portalProfileProviderData) : base(portalProfileProviderData)
        {
            this.portalProfileProviderData = portalProfileProviderData;
            this.onDataMappingRemoved = new ConfigurationNodeChangedEventHandler(this.OnDataMappingRemoved);
            this.onDataMappingRenamed = new ConfigurationNodeChangedEventHandler(this.OnDataMappingRenamed);
        }

        /// <summary>
        /// Gets or sets the database instance that will be used for the profile.
        /// </summary>
        /// <value>An <see cref="InstanceNode"/> object.</value>
        [Editor(typeof(ReferenceEditor), typeof(UITypeEditor))]
        [ReferenceType(typeof(DataSetMappingNode))]
        [SRDescription(SR.Keys.PortalProfileProviderPropertyDescription)]
        [Required]
        [SRCategory(SR.Keys.CategoryGeneral)]
        public DataSetMappingNode DataMapping
        {
            get { return this.mapper; }
            set
            {
                ILinkNodeService service = GetService(typeof(ILinkNodeService)) as ILinkNodeService;
                Debug.Assert(service != null, "Could not get the ILinkNodeService");
                this.mapper = (DataSetMappingNode)service.CreateReference(mapper, value, onDataMappingRemoved, onDataMappingRenamed);
                this.portalProfileProviderData.Mapping = string.Empty;
                if (this.mapper != null)
                {
                    this.portalProfileProviderData.Mapping = this.mapper.Name;
                }
            }
        }

        /// <summary>
        /// Resolves the database node reference.
        /// </summary>
        public override void ResolveNodeReferences()
        {
            base.ResolveNodeReferences();
            DataMappingsSettingsNode dataMappingsSettingsNode = Hierarchy.FindNodeByType(typeof(DataMappingsSettingsNode)) as DataMappingsSettingsNode;
            Debug.Assert(dataMappingsSettingsNode != null, "DataMappings Settings are missing.");
            this.DataMapping = Hierarchy.FindNodeByName(dataMappingsSettingsNode, this.portalProfileProviderData.Mapping) as DataSetMappingNode;
        }

        /// <summary>
        /// See <see cref="ProfileProviderNode.TypeName"/>.
        /// </summary>
        [Browsable(false)]
        public override string TypeName
        {
            get { return typeof(PortalProfileProvider).AssemblyQualifiedName; }
        }


        /// <summary>
        /// <para>Adds the default nodes that are needed for a <see cref="DbProfileProvider"/>. It creates the a node for the Data Application Block (<see cref="DatabaseSettingsNode"/>).</para>
        /// </summary>
        protected override void AddDefaultChildNodes()
        {
            base.AddDefaultChildNodes();
            CreateDataMappingsSettingsNode();
        }

        private void OnDataMappingRemoved(object sender, ConfigurationNodeChangedEventArgs args)
        {
            this.DataMapping = null;
        }

        private void OnDataMappingRenamed(object sender, ConfigurationNodeChangedEventArgs args)
        {
            this.portalProfileProviderData.Mapping = args.Node.Name;
        }

        private void CreateDataMappingsSettingsNode()
        {
            if (!DataSetMappingSettingsNodeExists())
            {
                AddConfigurationSectionCommand cmd = new AddConfigurationSectionCommand(Site, typeof(DataMappingsSettingsNode), DataMappingSettings.SectionName);
                cmd.Execute(Hierarchy.RootNode);
            }
        }

        private bool DataSetMappingSettingsNodeExists()
        {
			DataMappingsSettingsNode node = Hierarchy.FindNodeByType(typeof(DataMappingsSettingsNode)) as DataMappingsSettingsNode;
            return (node != null);
        }
    }
}