using System;
using System.Data;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Database;
using Microsoft.Practices.EnterpriseLibrary.Security.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Instrumentation;

using EAF.Security.PortalProfile.Configuration;

namespace EAF.Security.PortalProfile
{
    /// <summary>
    /// Represents a profile provider for the portal starter site.
    /// </summary>
    public class PortalProfileProvider  : ConfigurationProvider, IProfileProvider
    {
		private PortalProfileMapper portalProfileMapper;

        private SecurityConfigurationView securityConfigurationView;

        /// <summary>
        /// <para>Initialize a new instance of the <see cref="DbProfileProvider"/> class.</para>
        /// </summary>
        public PortalProfileProvider()
        {
        }

        /// <summary>
        /// <para>Initializes the provider with a <see cref="SecurityConfigurationView"/>.</para>
        /// </summary>
        /// <param name="configurationView">
        /// <para>A <see cref="SecurityConfigurationView"/> object.</para>
        /// </param>        
        public override void Initialize(ConfigurationView configurationView)
        {
            ArgumentValidation.CheckForNullReference(configurationView, "configurationView");
            ArgumentValidation.CheckExpectedType(configurationView, typeof(SecurityConfigurationView));

            this.securityConfigurationView = (SecurityConfigurationView)configurationView;

			portalProfileMapper = new PortalProfileMapper();
        }

        /// <summary>
        /// Persist a user's profile to the database.
        /// </summary>
        /// <param name="identity">An authenticated user's identity with a username that 
        /// matches a user in the Identity table.</param>
        /// <param name="profile">Object that represents the user's profile.  
        /// A single primitive, serializable object, or a dictionary of primitives and objects.</param>
        public void SetProfile(IIdentity identity, object profile)
        {
            ArgumentValidation.CheckForNullReference(identity, "identity");
            ArgumentValidation.CheckForNullReference(profile, "profile");
			ArgumentValidation.CheckExpectedType(profile,typeof(PortalProfile));

			PortalProfile thisProfile = profile as PortalProfile;

			portalProfileMapper.SetProfile(identity.Name, thisProfile);

            SecurityProfileSaveEvent.Fire(identity.Name);
        }

        public object GetProfile(IIdentity identity)
        {
            ArgumentValidation.CheckForNullReference(identity, "identity");

			PortalProfile dsPortalProfile = portalProfileMapper.GetProfile(identity.Name);

			//securityDb.LoadDataSet(SPGetProfile, dsPortalProfile, new string[] {"PortalProfiles"}, identity.Name);
			if (dsPortalProfile != null)
				ArgumentValidation.CheckExpectedType(dsPortalProfile,typeof(PortalProfile));
			
			PortalProfile profile = dsPortalProfile as PortalProfile;

            SecurityProfileLoadEvent.Fire(identity.Name);

            return profile;
        }
    }
}