using System;
using System.Diagnostics;
using System.Data;
using System.Data.Common;
using System.Xml;


namespace XmlFileProvider
{
	/// <summary>
	/// Summary description for XmlFileDataAdapter.
	/// </summary>
	public class XmlFileDataAdapter : DbDataAdapter, IDbDataAdapter
	{
		#region "private member variables"
			XmlFileCommand selectCommand;
			XmlFileCommand insertCommand;
			XmlFileCommand updateCommand;
			XmlFileCommand deleteCommand;
			private bool disposed;
		#endregion

		#region EVENTS
			private static readonly object EventRowUpdated = new object(); 
			private static readonly object EventRowUpdating = new object(); 
		#endregion

		
		public XmlFileDataAdapter() 
		{
			disposed = false;
		}

		public XmlFileDataAdapter(XmlFileCommand cmd)  : this()
		{
			selectCommand = cmd;
		}

		public XmlFileDataAdapter(String strCommand, XmlFileConnection conn) : this()
		{
			selectCommand = new XmlFileCommand(strCommand, conn);
		}

	
		#region IDbDataAdapter Members

		public IDbCommand UpdateCommand
		{
			get { return updateCommand; }
			set
			{
				if (value.GetType() == typeof(XmlFileCommand))
					updateCommand = (XmlFileCommand)value;
				else					
					throw new ApplicationException(SR.ExceptionMustBeXmlFileCmmands);
			}
		}

		public IDbCommand SelectCommand
		{
			get { return selectCommand; }
			set
			{
				if (value.GetType() == typeof(XmlFileCommand))
					selectCommand = (XmlFileCommand)value;
				else					
					throw new ApplicationException(SR.ExceptionMustBeXmlFileCmmands);
			}
		}

		public IDbCommand DeleteCommand
		{
			get { return deleteCommand; }
			set
			{
				if (value.GetType() == typeof(XmlFileCommand))
					deleteCommand = (XmlFileCommand)value;
				else					
					throw new ApplicationException(SR.ExceptionMustBeXmlFileCmmands);
			}
		}

		public IDbCommand InsertCommand
		{
			get { return insertCommand; }
			set
			{
				if (value.GetType() == typeof(XmlFileCommand))
					insertCommand = (XmlFileCommand)value;
				else					
					throw new ApplicationException(SR.ExceptionMustBeXmlFileCmmands);
			}
		}

		#endregion

		#region IDataAdapter Members

		public new int Update(DataSet dataSet)
		{
			dataSet.WriteXml(this.selectCommand.CommandText);
			return 0;
		}

		public new int Update(DataSet dataSet, string srcTable)
		{
			if (dataSet.Tables[srcTable] == dataSet.Tables[dataSet.Tables.Count])
				dataSet.WriteXml(this.selectCommand.CommandText);
			return 0;
		}
			
		override protected RowUpdatedEventArgs CreateRowUpdatedEvent(DataRow dataRow, IDbCommand command, StatementType statementType, DataTableMapping tableMapping)
		{
			return new XmlFileRowUpdatedEventArgs(dataRow, command, statementType, tableMapping);
		}

		override protected RowUpdatingEventArgs CreateRowUpdatingEvent(DataRow dataRow, IDbCommand command, StatementType statementType, DataTableMapping tableMapping)
		{
			return new XmlFileRowUpdatingEventArgs(dataRow, command, statementType, tableMapping);
		}

		override protected void OnRowUpdating(RowUpdatingEventArgs value)
		{
			XmlFileRowUpdatingEventHandler handler = (XmlFileRowUpdatingEventHandler) Events[EventRowUpdating];
			if ((null != handler) && (value is XmlFileRowUpdatingEventArgs)) 
			{
				handler(this, (XmlFileRowUpdatingEventArgs) value);
			}
		}

		override protected void OnRowUpdated(RowUpdatedEventArgs value)
		{
			XmlFileRowUpdatedEventHandler handler = (XmlFileRowUpdatedEventHandler) Events[EventRowUpdated];
			if ((null != handler) && (value is XmlFileRowUpdatedEventArgs)) 
			{
				handler(this, (XmlFileRowUpdatedEventArgs) value);
			}
		}
		
		public event XmlFileRowUpdatingEventHandler RowUpdating
		{
			add { Events.AddHandler(EventRowUpdating, value); }
			remove { Events.RemoveHandler(EventRowUpdating, value); }
		}

		public event XmlFileRowUpdatedEventHandler RowUpdated
		{
			add { Events.AddHandler(EventRowUpdated, value); }
			remove { Events.RemoveHandler(EventRowUpdated, value); }
		}

		#endregion
	
		protected override void Dispose(bool disposing)
		{
			if (!disposed)
				try
				{
					if (disposing)
					{
						if (SelectCommand != null)
						{
							this.SelectCommand.Dispose();
						}
						if (InsertCommand != null)
						{
							this.InsertCommand.Dispose();
						}
						if (UpdateCommand != null)
						{
							this.UpdateCommand.Dispose();
						}
					} //end if

					disposed = true;
				} //end try
				finally 
				{
				} //end finally
		}  //end Dispose
	
		protected override void OnFillError(FillErrorEventArgs value)
		{
			base.OnFillError (value);
		}
	
		protected override int Update(DataRow[] dataRows, DataTableMapping tableMapping)
		{
			if (dataRows.Length <= 0) return 0;
 
			DataTable dtTable = dataRows[0].Table;
			DataSet dsDataSet = dtTable.DataSet;
			if (dtTable == dsDataSet.Tables[(dsDataSet.Tables.Count-1)])
				dsDataSet.WriteXml(this.insertCommand.Connection.ConnectionString);
			return base.Update (dataRows, tableMapping);
		}
	}

	public delegate void XmlFileRowUpdatingEventHandler(object sender, XmlFileRowUpdatingEventArgs e);
	public delegate void XmlFileRowUpdatedEventHandler(object sender, XmlFileRowUpdatedEventArgs e);

	public class XmlFileRowUpdatingEventArgs : RowUpdatingEventArgs
	{
		public XmlFileRowUpdatingEventArgs(DataRow row, IDbCommand command, StatementType statementType, DataTableMapping tableMapping) 
			: base(row, command, statementType, tableMapping) 
		{
			Debug.WriteLine("XmlFileRowUpdatingEventArgs.Constructor", "XmlFileRowUpdatingEventArgs");
		}

		// Hide the inherited implementation of the command property.
		new public XmlFileCommand Command
		{
			get  { return (XmlFileCommand)base.Command; }
			set  { base.Command = value; }
		}
	}

	public class XmlFileRowUpdatedEventArgs : RowUpdatedEventArgs
	{
		public XmlFileRowUpdatedEventArgs(DataRow row, IDbCommand command, StatementType statementType, DataTableMapping tableMapping)
			: base(row, command, statementType, tableMapping) 
		{
			Debug.WriteLine("XmlFileRowUpdatedEventArgs.Constructor", "XmlFileRowUpdatedEventArgs");
		}

		// Hide the inherited implementation of the command property.
		new public XmlFileCommand Command
		{
			get  { return (XmlFileCommand)base.Command; }
		}
	}
}
