using System;
using System.Diagnostics;
using System.Collections;
using System.Data;
using System.Xml;

namespace XmlFileProvider
{
	/// <summary>
	/// Summary description for XmlFileReader.
	/// </summary>
	// This should implement IDataReader
	public class XmlFileDataReader : IDataReader
	{
		#region "Private member variables"
			private	XmlFileConnection xmlFileConnection = null;
			private	string fileName;	
			private DataSet dataSet = new DataSet();
			private int tableNum = 0;
			private int rowNum = -1;
			private bool closed = true;
		#endregion
		
		internal XmlFileDataReader() {}

		internal XmlFileDataReader(String command)
		{
			fileName = command;
			GetXmlFile(fileName);
		}

		internal XmlFileDataReader(String command, XmlFileConnection conn) : this(command)
		{
			xmlFileConnection = conn;
			if (xmlFileConnection != null) xmlFileConnection.ChangeDatabase(fileName);
		}

		private void NotSupported()
		{
			throw new System.NotSupportedException();
		}

		#region IDisposable Members

		public void Dispose()
		{
		}

		#endregion
	
		#region IDataReader Members

		public int RecordsAffected
		{
			get
			{
				int iRecNum = 0;
				foreach (DataTable dtTable in dataSet.Tables)
					iRecNum += dtTable.Rows.Count;

				return iRecNum;
			}
		}

		public bool IsClosed
		{
			get
			{
				return closed;
			}
		}

		public bool NextResult()
		{
			return (tableNum < dataSet.Tables.Count);
		}

		public void Close()
		{
			closed = true;
			if (xmlFileConnection != null) xmlFileConnection.Close();
			
		}

		internal void GetXmlFile(string command)
		{
			fileName = command;
			
			try
			{
				dataSet.ReadXml(command);
				rowNum = -1;
				closed = false;
			}
			catch (Exception ex)
			{
				throw new ApplicationException(SR.ExceptionLoadXMLFile(command, ex.Message));					
			}
		}

		internal XmlReader XmlReader
		{
			get
			{
				try
				{
					XmlReader retVal = new XmlTextReader(new System.IO.StringReader(dataSet.GetXml()));
					return retVal;
				}
				catch (Exception ex)
				{
					throw new ApplicationException(SR.ExceptionCreateReader(ex.Message));
				}
			}
		}

		public bool Read()
		{
			Debug.WriteLine("XmlFileDataReader.Read", "XmlFileDataReader");
			bool bRetVal = false;
			if (closed) return bRetVal;
			if (rowNum < (dataSet.Tables[tableNum].Rows.Count -1))
			{
				bRetVal = true;
				rowNum++;
			}
			else
			{
				tableNum++;
				rowNum = -1;
			}
			return bRetVal;
		}

		public int Depth
		{
			get
			{
				return 0;
			}
		}

		public DataTable GetSchemaTable()
		{
			DataTable dtTable = new DataTable();

			// all of common, non "base-table" fields implemented
			dtTable.Columns.Add("ColumnName", typeof(System.String));
			dtTable.Columns.Add("ColumnSize", typeof(Int32));
			dtTable.Columns.Add("ColumnOrdinal", typeof(Int32));
			dtTable.Columns.Add("NumericPrecision", typeof(Int16));
			dtTable.Columns.Add("NumericScale", typeof(Int16));
			dtTable.Columns.Add("DataType", typeof(System.Type));
			dtTable.Columns.Add("AllowDBNull", typeof(bool));
			dtTable.Columns.Add("IsReadOnly", typeof(bool));
			dtTable.Columns.Add("IsUnique", typeof(bool));
			dtTable.Columns.Add("IsRowVersion", typeof(bool));
			dtTable.Columns.Add("IsKey", typeof(bool));
			dtTable.Columns.Add("IsAutoIncrement", typeof(bool));
			dtTable.Columns.Add("IsLong", typeof(bool));

			foreach (DataColumn dcColumn in dataSet.Tables[tableNum].Columns)
			{
				DataRow r = dtTable.NewRow();
				r["ColumnName"] = dcColumn.ColumnName;
				r["ColumnSize"] = dcColumn.MaxLength;
				r["ColumnOrdinal"] = dcColumn.Ordinal;
				r["NumericPrecision"] = 0;
				r["NumericScale"] = 0;
				r["DataType"] = dcColumn.DataType;
				r["AllowDBNull"] = dcColumn.AllowDBNull;
				r["IsReadOnly"] = dcColumn.ReadOnly;
				r["IsUnique"] = dcColumn.Unique;
				r["IsRowVersion"] = false;
				r["IsKey"] = false;
				r["IsAutoIncrement"] = dcColumn.AutoIncrement;
				r["IsLong"] = false;

				dtTable.Rows.Add(r);
			}
			return dtTable;
		}

		#endregion

		#region IDataRecord Members

		public int GetInt32(int i)
		{
			return (Int32)GetValue(i);
		}

		public object this[string name]
		{
			get
			{
				return dataSet.Tables[tableNum].Rows[rowNum][name];
			}
		}

		object System.Data.IDataRecord.this[int i]
		{
			get
			{
				return GetValue(i);
			}
		}

		public object GetValue(int i)
		{
			return dataSet.Tables[tableNum].Rows[rowNum][i];
		}

		public bool IsDBNull(int i)
		{
			return (GetValue(i).GetType().Equals(typeof(System.DBNull)));
		}

		public long GetBytes(int i, long fieldOffset, byte[] buffer, int bufferoffset, int length)
		{
			// TODO:  Add XmlFileDataReader.GetBytes implementation
			return 0;
		}

		public byte GetByte(int i)
		{
			return (Byte)GetValue(i);
		}

		public Type GetFieldType(int i)
		{
			Type tFieldType = dataSet.Tables[tableNum].Columns[i].DataType;
			return tFieldType;
			//return null;
		}

		public decimal GetDecimal(int i)
		{
			return (Decimal)GetValue(i);
		}

		public int GetValues(object[] values)
		{
			object[] valArray = dataSet.Tables[tableNum].Rows[rowNum].ItemArray;
			Array.Copy(valArray,values,valArray.Length);
			return valArray.Length;
		}

		public string GetName(int i)
		{
			string strName = dataSet.Tables[tableNum].Columns[i].ColumnName;
			return strName;
		}

		public int FieldCount
		{
			get
			{
				return dataSet.Tables[tableNum].Columns.Count;
			}
		}

		public long GetInt64(int i)
		{
			return (Int64)GetValue(i);
		}

		public double GetDouble(int i)
		{
			return (Double)GetValue(i);
		}

		public bool GetBoolean(int i)
		{
			return (bool)GetValue(i);
		}

		public Guid GetGuid(int i)
		{
			return (Guid)GetValue(i);
		}

		public DateTime GetDateTime(int i)
		{
			return (DateTime)GetValue(i);
		}

		public int GetOrdinal(string name)
		{
			return Array.IndexOf(dataSet.Tables[tableNum].Rows[rowNum].ItemArray, name);
		}

		public string GetDataTypeName(int i)
		{
			return GetFieldType(i).ToString();
		}

		public float GetFloat(int i)
		{
			return (float)GetValue(i);
		}

		public IDataReader GetData(int i)
		{
			NotSupported();
			return null;
		}

		public long GetChars(int i, long fieldoffset, char[] buffer, int bufferoffset, int length)
		{
			NotSupported();
			return 0;	
		}

		public string GetString(int i)
		{
			return (string)GetValue(i);
		}

		public char GetChar(int i)
		{
			return (char)GetChar(i);
		}

		public short GetInt16(int i)
		{
			return (short)GetValue(i);
		}

		#endregion
	}
}
